# Copyright (c) 2023 Binbin Zhang (binbzha@qq.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

from wenet.cli.paraformer_model import load_model as load_paraformer
from wenet.cli.model import load_model


def get_args():
    parser = argparse.ArgumentParser(description='')
    parser.add_argument('audio_file', help='audio file to transcribe')
    parser.add_argument('-l',
                        '--language',
                        choices=[
                            'chinese',
                            'english',
                        ],
                        default='chinese',
                        help='language type')
    parser.add_argument('-m',
                        '--model_dir',
                        default=None,
                        help='specify your own model dir')
    parser.add_argument('-g',
                        '--gpu',
                        type=int,
                        default='-1',
                        help='gpu id to decode, default is cpu.')
    parser.add_argument('-t',
                        '--show_tokens_info',
                        action='store_true',
                        help='whether to output token(word) level information'
                        ', such times/confidence')
    parser.add_argument('--align',
                        action='store_true',
                        help='force align the input audio and transcript')
    parser.add_argument('--label', type=str, help='the input label to align')
    parser.add_argument('--paraformer',
                        action='store_true',
                        help='whether to use the best chinese model')
    parser.add_argument('--beam', type=int, default=5, help="beam size")
    parser.add_argument('--context_path',
                        type=str,
                        default=None,
                        help='context list file')
    parser.add_argument('--context_score',
                        type=float,
                        default=6.0,
                        help='context score')
    args = parser.parse_args()
    return args


def main():
    args = get_args()

    if args.paraformer:
        model = load_paraformer(args.model_dir, args.gpu)
    else:
        model = load_model(args.language, args.model_dir, args.gpu, args.beam,
                           args.context_path, args.context_score)
    if args.align:
        result = model.align(args.audio_file, args.label)
    else:
        result = model.transcribe(args.audio_file, args.show_tokens_info)
    print(result)


if __name__ == "__main__":
    main()



from os import PathLike
from typing import Dict, List, Optional, Tuple, Union
from wenet.text.base_tokenizer import BaseTokenizer

from wenet.utils.file_utils import read_non_lang_symbols


class WhisperTokenizer(BaseTokenizer):

    def __init__(
        self,
        multilingual: bool,
        num_languages: int = 99,
        language: Optional[str] = None,
        task: Optional[str] = None,
        non_lang_syms: Optional[Union[str, PathLike, List]] = None,
        *args,
        **kwargs,
    ) -> None:
        # NOTE(Mddct): don't build here, pickle issues
        self.tokenizer = None
        # TODO: we don't need this in future
        self.multilingual = multilingual
        self.num_languages = num_languages
        self.language = language
        self.task = task

        if not isinstance(non_lang_syms, List):
            self.non_lang_syms = read_non_lang_symbols(non_lang_syms)
        else:
            # non_lang_syms=["{NOISE}"]
            self.non_lang_syms = non_lang_syms
        # TODO(Mddct): add special tokens, like non_lang_syms
        del self.non_lang_syms

    def __getstate__(self):
        state = self.__dict__.copy()
        del state['tokenizer']
        return state

    def __setstate__(self, state):
        self.__dict__.update(state)
        recovery = {'tokenizer': None}
        self.__dict__.update(recovery)

    def _build_tiktoken(self):
        if self.tokenizer is None:
            from whisper.tokenizer import get_tokenizer
            self.tokenizer = get_tokenizer(multilingual=self.multilingual,
                                           num_languages=self.num_languages,
                                           language=self.language,
                                           task=self.task)
            self.t2i = {}
            self.i2t = {}
            for i in range(self.tokenizer.encoding.n_vocab):
                unit = str(
                    self.tokenizer.encoding.decode_single_token_bytes(i))
                if len(unit) == 0:
                    unit = str(i)
                unit = unit.replace(" ", "<space>")
                # unit = bytes(unit, 'utf-8')
                self.t2i[unit] = i
                self.i2t[i] = unit
            assert len(self.t2i) == len(self.i2t)

    def tokenize(self, line: str) -> Tuple[List[str], List[int]]:
        self._build_tiktoken()
        ids = self.tokenizer.encoding.encode(line)
        text = [self.i2t[d] for d in ids]
        return text, ids

    def detokenize(self, ids: List[int]) -> Tuple[str, List[str]]:
        self._build_tiktoken()
        tokens = [self.i2t[d] for d in ids]
        text = self.tokenizer.encoding.decode(ids)
        return text, tokens

    def text2tokens(self, line: str) -> List[str]:
        self._build_tiktoken()
        return self.tokenize(line)[0]

    def tokens2text(self, tokens: List[str]) -> str:
        self._build_tiktoken()
        ids = [self.t2i[t] for t in tokens]
        return self.detokenize(ids)[0]

    def tokens2ids(self, tokens: List[str]) -> List[int]:
        self._build_tiktoken()
        ids = [self.t2i[t] for t in tokens]
        return ids

    def ids2tokens(self, ids: List[int]) -> List[str]:
        self._build_tiktoken()
        return [self.tokenizer.encoding.decode([id]) for id in ids]

    def vocab_size(self) -> int:
        self._build_tiktoken()
        return len(self.t2i)

    @property
    def symbol_table(self) -> Dict[str, int]:
        self._build_tiktoken()
        return self.t2i



import os
import subprocess


def do_download_from_play_url(input_url, output_dir, wav_type='mp3', wav_name='loaded_audio', keep_title=False, is_list=False):
    """

    :param input_url:
    :param output_dir:
    :param wav_type: mp3 or wav
    :param wav_name:
    :param keep_title: 是不是要把标题保留到音频名称内
    :param is_list:  如果url为一个列表页面, 也就是那种一个音频在播放, 但是旁边有个系列列表(不是推荐视频列表).然后把此开启就可以自动下载整个列表
    :return:
    """
    template_dir = os.path.join(output_dir, wav_name)
    if keep_title:
        template_dir = f"{template_dir}_%(title)s.%(ext)s"
        template_dir = template_dir + "_%(title)s"
    if is_list:
        template_dir = template_dir + '_%(id)s'
    template_dir = template_dir + ".%(ext)s"
    if wav_type.startswith('.'):
        wav_type = wav_type[1:]
    command = [
        'yt-dlp',
        '-x',  # 仅提取音频
        '--audio-format', wav_type,  # 设置音频格式为wav,mp3等
        '--output', template_dir,  # 设置输出文件名模板
        input_url  # YouTube视频URL
    ]
    print(f'开始下载, link: {input_url}')
    res = subprocess.run(command, capture_output=True, text=True)
    res = str(res)[-100:]
    print(f"下载完成，link: {input_url}\n res:", res)

if __name__ == '__main__':
    do_download_from_play_url(input_url="https://www.youtube.com/watch?v=-oQNbCVmcrw", output_dir="./download")
    do_download_from_play_url(input_url="https://www.youtube.com/watch?v=kJeTPi7qJXw", output_dir="./download")


# Copyright (c) 2020 Mobvoi Inc. (authors: Binbin Zhang)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import re

import yaml
import torch
from collections import OrderedDict

import datetime


def load_checkpoint(model: torch.nn.Module, path: str) -> dict:
    logging.info('Checkpoint: loading from checkpoint %s' % path)
    checkpoint = torch.load(path, map_location='cpu')
    missing_keys, unexpected_keys = model.load_state_dict(checkpoint,
                                                          strict=False)
    for key in missing_keys:
        logging.info("missing tensor: {}".format(key))
    for key in unexpected_keys:
        logging.info("unexpected tensor: {}".format(key))
    info_path = re.sub('.pt$', '.yaml', path)
    configs = {}
    if os.path.exists(info_path):
        with open(info_path, 'r') as fin:
            configs = yaml.load(fin, Loader=yaml.FullLoader)
    return configs


def save_checkpoint(model: torch.nn.Module, path: str, infos=None):
    '''
    Args:
        infos (dict or None): any info you want to save.
    '''
    logging.info('Checkpoint: save to checkpoint %s' % path)
    if isinstance(model, torch.nn.DataParallel):
        state_dict = model.module.state_dict()
    elif isinstance(model, torch.nn.parallel.DistributedDataParallel):
        state_dict = model.module.state_dict()
    else:
        state_dict = model.state_dict()
    torch.save(state_dict, path)
    info_path = re.sub('.pt$', '.yaml', path)
    if infos is None:
        infos = {}
    infos['save_time'] = datetime.datetime.now().strftime('%d/%m/%Y %H:%M:%S')
    with open(info_path, 'w') as fout:
        data = yaml.dump(infos)
        fout.write(data)


def filter_modules(model_state_dict, modules):
    new_mods = []
    incorrect_mods = []
    mods_model = model_state_dict.keys()
    for mod in modules:
        if any(key.startswith(mod) for key in mods_model):
            new_mods += [mod]
        else:
            incorrect_mods += [mod]
    if incorrect_mods:
        logging.warning(
            "module(s) %s don't match or (partially match) "
            "available modules in model.",
            incorrect_mods,
        )
        logging.warning("for information, the existing modules in model are:")
        logging.warning("%s", mods_model)

    return new_mods


def load_trained_modules(model: torch.nn.Module, args: None):
    # Load encoder modules with pre-trained model(s).
    enc_model_path = args.enc_init
    enc_modules = args.enc_init_mods
    main_state_dict = model.state_dict()
    logging.warning("model(s) found for pre-initialization")
    if os.path.isfile(enc_model_path):
        logging.info('Checkpoint: loading from checkpoint %s for CPU' %
                     enc_model_path)
        model_state_dict = torch.load(enc_model_path, map_location='cpu')
        modules = filter_modules(model_state_dict, enc_modules)
        partial_state_dict = OrderedDict()
        for key, value in model_state_dict.items():
            if any(key.startswith(m) for m in modules):
                partial_state_dict[key] = value
        main_state_dict.update(partial_state_dict)
    else:
        logging.warning("model was not found : %s", enc_model_path)

    model.load_state_dict(main_state_dict)
    configs = {}
    return configs



# Copyright (c) 2022 Ximalaya Inc. (authors: Yuguang Yang)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Conv2d Module with Valid Padding"""

import torch.nn.functional as F
from torch.nn.modules.conv import _ConvNd, _size_2_t, Union, _pair, Tensor, Optional


class Conv2dValid(_ConvNd):
    """
    Conv2d operator for VALID mode padding.
    """

    def __init__(
            self,
            in_channels: int,
            out_channels: int,
            kernel_size: _size_2_t,
            stride: _size_2_t = 1,
            padding: Union[str, _size_2_t] = 0,
            dilation: _size_2_t = 1,
            groups: int = 1,
            bias: bool = True,
            padding_mode: str = 'zeros',  # TODO: refine this type
            device=None,
            dtype=None,
            valid_trigx: bool = False,
            valid_trigy: bool = False) -> None:
        factory_kwargs = {'device': device, 'dtype': dtype}
        kernel_size_ = _pair(kernel_size)
        stride_ = _pair(stride)
        padding_ = padding if isinstance(padding, str) else _pair(padding)
        dilation_ = _pair(dilation)
        super(Conv2dValid,
              self).__init__(in_channels, out_channels,
                             kernel_size_, stride_, padding_, dilation_, False,
                             _pair(0), groups, bias, padding_mode,
                             **factory_kwargs)
        self.valid_trigx = valid_trigx
        self.valid_trigy = valid_trigy

    def _conv_forward(self, input: Tensor, weight: Tensor,
                      bias: Optional[Tensor]):
        validx, validy = 0, 0
        if self.valid_trigx:
            validx = (input.size(-2) *
                      (self.stride[-2] - 1) - 1 + self.kernel_size[-2]) // 2
        if self.valid_trigy:
            validy = (input.size(-1) *
                      (self.stride[-1] - 1) - 1 + self.kernel_size[-1]) // 2
        return F.conv2d(input, weight, bias, self.stride, (validx, validy),
                        self.dilation, self.groups)

    def forward(self, input: Tensor) -> Tensor:
        return self._conv_forward(input, self.weight, self.bias)



import ChatTTS
import torch
import torchaudio


from gxl_ai_utils.utils import utils_file
chat = ChatTTS.Chat()
chat.load(compile=False) # Set to True for better performance
OUTPUT_DIR='/home/work_nfs15/asr_data/data/chat_data/wav'
def main(texts, keys):
    batch_size = 5
    wavs = chat.infer(texts)
    for i in range(len(wavs)):
        """
        In some versions of torchaudio, the first line works but in other versions, so does the second line.
        """
        try:
            torchaudio.save(f"{OUTPUT_DIR}/{keys[i]}.wav", torch.from_numpy(wavs[i]).unsqueeze(0), 24000)
        except:
            torchaudio.save(f"{OUTPUT_DIR}/{keys[i]}.wav", torch.from_numpy(wavs[i]), 24000)

import os
import multiprocessing


def process_task(device_id, dict_list_temp):
    # 设置CUDA_VISIBLE_DEVICES，使得该进程只看到指定的GPU
    os.environ["CUDA_VISIBLE_DEVICES"] = str(device_id)

    texts = [x["Q"] for x in dict_list_temp]
    keys = [x["key"] for x in dict_list_temp]

    # 执行主要的TTS任务
    main(texts, keys)
    print(f"TTS application finished on GPU {device_id}.")


if __name__ == "__main__":
    input_jsonl_path = "/home/work_nfs15/asr_data/data/chat_data/test.jsonl"
    dict_list = utils_file.load_dict_list_from_jsonl(input_jsonl_path)

    little_thread_num = 8  # 总线程数
    num_device = 8  # GPU数量
    dict_list_list = utils_file.do_split_list(dict_list, little_thread_num)  # 拆分数据

    processes = []  # 存储进程的列表

    for thread_index in range(little_thread_num):
        # 计算当前线程分配的GPU ID（环绕分配）
        device_id = thread_index % num_device

        # 获取当前线程需要处理的数据
        dict_list_temp = dict_list_list[thread_index]

        # 创建一个新的进程来执行TTS任务
        p = multiprocessing.Process(target=process_task, args=(device_id, dict_list_temp))
        processes.append(p)
        p.start()

    # 等待所有进程结束
    for p in processes:
        p.join()

    print("All TTS applications finished.")




# isort:skip_file

from . import checkpoint, decode, dist, env, utils

from .byte_utils import (
    byte_decode,
    byte_encode,
    smart_byte_decode,
)

from .checkpoint import (
    average_checkpoints,
    find_checkpoints,
    load_checkpoint,
    remove_checkpoints,
    save_checkpoint,
    save_checkpoint_with_global_batch_idx,
)

from .context_graph import ContextGraph, ContextState

from .decode import (
    get_lattice,
    nbest_decoding,
    nbest_oracle,
    one_best_decoding,
    rescore_with_attention_decoder,
    rescore_with_n_best_list,
    rescore_with_whole_lattice,
)

from .dist import (
    cleanup_dist,
    setup_dist,
)

from .env import (
    get_env_info,
    get_git_branch_name,
    get_git_date,
    get_git_sha1,
)

from .utils import (
    AttributeDict,
    MetricsTracker,
    add_eos,
    add_sos,
    concat,
    encode_supervisions,
    get_alignments,
    get_executor,
    get_texts,
    is_cjk,
    is_jit_tracing,
    is_module_available,
    l1_norm,
    l2_norm,
    linf_norm,
    load_alignments,
    make_pad_mask,
    measure_gradient_norms,
    measure_weight_norms,
    optim_step_and_measure_param_change,
    save_alignments,
    setup_logger,
    store_transcripts,
    str2bool,
    subsequent_chunk_mask,
    tokenize_by_CJK_char,
    write_error_stats,
)

from .ngram_lm import NgramLm, NgramLmStateCost

from .lm_wrapper import LmScorer



#!/usr/bin/env python3
# Copyright      2023  Xiaomi Corp.        (authors: Fangjun Kuang)
# Copyright      2023  Danqing Fu (danqing.fu@gmail.com)

"""
This script loads ONNX models exported by ./export-onnx-streaming.py
and uses them to decode waves.

We use the pre-trained model from
https://huggingface.co/Zengwei/icefall-asr-librispeech-streaming-zipformer-2023-05-17
as an example to show how to use this file.

1. Download the pre-trained model

cd egs/librispeech/ASR

repo_url=https://huggingface.co/Zengwei/icefall-asr-librispeech-streaming-zipformer-2023-05-17
GIT_LFS_SKIP_SMUDGE=1 git clone $repo_url
repo=$(basename $repo_url)

pushd $repo
git lfs pull --include "exp/pretrained.pt"

cd exp
ln -s pretrained.pt epoch-99.pt
popd

2. Export the model to ONNX

./zipformer/export-onnx-streaming.py \
  --tokens $repo/data/lang_bpe_500/tokens.txt \
  --use-averaged-model 0 \
  --epoch 99 \
  --avg 1 \
  --exp-dir $repo/exp \
  --num-encoder-layers "2,2,3,4,3,2" \
  --downsampling-factor "1,2,4,8,4,2" \
  --feedforward-dim "512,768,1024,1536,1024,768" \
  --num-heads "4,4,4,8,4,4" \
  --encoder-dim "192,256,384,512,384,256" \
  --query-head-dim 32 \
  --value-head-dim 12 \
  --pos-head-dim 4 \
  --pos-dim 48 \
  --encoder-unmasked-dim "192,192,256,256,256,192" \
  --cnn-module-kernel "31,31,15,15,15,31" \
  --decoder-dim 512 \
  --joiner-dim 512 \
  --causal True \
  --chunk-size 16 \
  --left-context-frames 64

It will generate the following 3 files inside $repo/exp:

  - encoder-epoch-99-avg-1.onnx
  - decoder-epoch-99-avg-1.onnx
  - joiner-epoch-99-avg-1.onnx

3. Run this file with the exported ONNX models

./zipformer/onnx_pretrained-streaming.py \
  --encoder-model-filename $repo/exp/encoder-epoch-99-avg-1.onnx \
  --decoder-model-filename $repo/exp/decoder-epoch-99-avg-1.onnx \
  --joiner-model-filename $repo/exp/joiner-epoch-99-avg-1.onnx \
  --tokens $repo/data/lang_bpe_500/tokens.txt \
  $repo/test_wavs/1089-134686-0001.wav

Note: Even though this script only supports decoding a single file,
the exported ONNX models do support batch processing.
"""

import argparse
import logging
from typing import Dict, List, Optional, Tuple

import k2
import numpy as np
import onnxruntime as ort
import torch
import torchaudio
from kaldifeat import FbankOptions, OnlineFbank, OnlineFeature


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "--encoder-model-filename",
        type=str,
        required=True,
        help="Path to the encoder onnx model. ",
    )

    parser.add_argument(
        "--decoder-model-filename",
        type=str,
        required=True,
        help="Path to the decoder onnx model. ",
    )

    parser.add_argument(
        "--joiner-model-filename",
        type=str,
        required=True,
        help="Path to the joiner onnx model. ",
    )

    parser.add_argument(
        "--tokens",
        type=str,
        help="""Path to tokens.txt.""",
    )

    parser.add_argument(
        "sound_file",
        type=str,
        help="The input sound file to transcribe. "
        "Supported formats are those supported by torchaudio.load(). "
        "For example, wav and flac are supported. "
        "The sample rate has to be 16kHz.",
    )

    return parser


class OnnxModel:
    def __init__(
        self,
        encoder_model_filename: str,
        decoder_model_filename: str,
        joiner_model_filename: str,
    ):
        session_opts = ort.SessionOptions()
        session_opts.inter_op_num_threads = 1
        session_opts.intra_op_num_threads = 1

        self.session_opts = session_opts

        self.init_encoder(encoder_model_filename)
        self.init_decoder(decoder_model_filename)
        self.init_joiner(joiner_model_filename)

    def init_encoder(self, encoder_model_filename: str):
        self.encoder = ort.InferenceSession(
            encoder_model_filename,
            sess_options=self.session_opts,
            providers=["CPUExecutionProvider"],
        )
        self.init_encoder_states()

    def init_encoder_states(self, batch_size: int = 1):
        encoder_meta = self.encoder.get_modelmeta().custom_metadata_map
        logging.info(f"encoder_meta={encoder_meta}")

        model_type = encoder_meta["model_type"]
        assert model_type == "zipformer2", model_type

        decode_chunk_len = int(encoder_meta["decode_chunk_len"])
        T = int(encoder_meta["T"])

        num_encoder_layers = encoder_meta["num_encoder_layers"]
        encoder_dims = encoder_meta["encoder_dims"]
        cnn_module_kernels = encoder_meta["cnn_module_kernels"]
        left_context_len = encoder_meta["left_context_len"]
        query_head_dims = encoder_meta["query_head_dims"]
        value_head_dims = encoder_meta["value_head_dims"]
        num_heads = encoder_meta["num_heads"]

        def to_int_list(s):
            return list(map(int, s.split(",")))

        num_encoder_layers = to_int_list(num_encoder_layers)
        encoder_dims = to_int_list(encoder_dims)
        cnn_module_kernels = to_int_list(cnn_module_kernels)
        left_context_len = to_int_list(left_context_len)
        query_head_dims = to_int_list(query_head_dims)
        value_head_dims = to_int_list(value_head_dims)
        num_heads = to_int_list(num_heads)

        logging.info(f"decode_chunk_len: {decode_chunk_len}")
        logging.info(f"T: {T}")
        logging.info(f"num_encoder_layers: {num_encoder_layers}")
        logging.info(f"encoder_dims: {encoder_dims}")
        logging.info(f"cnn_module_kernels: {cnn_module_kernels}")
        logging.info(f"left_context_len: {left_context_len}")
        logging.info(f"query_head_dims: {query_head_dims}")
        logging.info(f"value_head_dims: {value_head_dims}")
        logging.info(f"num_heads: {num_heads}")

        num_encoders = len(num_encoder_layers)

        self.states = []
        for i in range(num_encoders):
            num_layers = num_encoder_layers[i]
            key_dim = query_head_dims[i] * num_heads[i]
            embed_dim = encoder_dims[i]
            nonlin_attn_head_dim = 3 * embed_dim // 4
            value_dim = value_head_dims[i] * num_heads[i]
            conv_left_pad = cnn_module_kernels[i] // 2

            for layer in range(num_layers):
                cached_key = torch.zeros(
                    left_context_len[i], batch_size, key_dim
                ).numpy()
                cached_nonlin_attn = torch.zeros(
                    1, batch_size, left_context_len[i], nonlin_attn_head_dim
                ).numpy()
                cached_val1 = torch.zeros(
                    left_context_len[i], batch_size, value_dim
                ).numpy()
                cached_val2 = torch.zeros(
                    left_context_len[i], batch_size, value_dim
                ).numpy()
                cached_conv1 = torch.zeros(batch_size, embed_dim, conv_left_pad).numpy()
                cached_conv2 = torch.zeros(batch_size, embed_dim, conv_left_pad).numpy()
                self.states += [
                    cached_key,
                    cached_nonlin_attn,
                    cached_val1,
                    cached_val2,
                    cached_conv1,
                    cached_conv2,
                ]
        embed_states = torch.zeros(batch_size, 128, 3, 19).numpy()
        self.states.append(embed_states)
        processed_lens = torch.zeros(batch_size, dtype=torch.int64).numpy()
        self.states.append(processed_lens)

        self.num_encoders = num_encoders

        self.segment = T
        self.offset = decode_chunk_len

    def init_decoder(self, decoder_model_filename: str):
        self.decoder = ort.InferenceSession(
            decoder_model_filename,
            sess_options=self.session_opts,
            providers=["CPUExecutionProvider"],
        )

        decoder_meta = self.decoder.get_modelmeta().custom_metadata_map
        self.context_size = int(decoder_meta["context_size"])
        self.vocab_size = int(decoder_meta["vocab_size"])

        logging.info(f"context_size: {self.context_size}")
        logging.info(f"vocab_size: {self.vocab_size}")

    def init_joiner(self, joiner_model_filename: str):
        self.joiner = ort.InferenceSession(
            joiner_model_filename,
            sess_options=self.session_opts,
            providers=["CPUExecutionProvider"],
        )

        joiner_meta = self.joiner.get_modelmeta().custom_metadata_map
        self.joiner_dim = int(joiner_meta["joiner_dim"])

        logging.info(f"joiner_dim: {self.joiner_dim}")

    def _build_encoder_input_output(
        self,
        x: torch.Tensor,
    ) -> Tuple[Dict[str, np.ndarray], List[str]]:
        encoder_input = {"x": x.numpy()}
        encoder_output = ["encoder_out"]

        def build_inputs_outputs(tensors, i):
            assert len(tensors) == 6, len(tensors)

            # (downsample_left, batch_size, key_dim)
            name = f"cached_key_{i}"
            encoder_input[name] = tensors[0]
            encoder_output.append(f"new_{name}")

            # (1, batch_size, downsample_left, nonlin_attn_head_dim)
            name = f"cached_nonlin_attn_{i}"
            encoder_input[name] = tensors[1]
            encoder_output.append(f"new_{name}")

            # (downsample_left, batch_size, value_dim)
            name = f"cached_val1_{i}"
            encoder_input[name] = tensors[2]
            encoder_output.append(f"new_{name}")

            # (downsample_left, batch_size, value_dim)
            name = f"cached_val2_{i}"
            encoder_input[name] = tensors[3]
            encoder_output.append(f"new_{name}")

            # (batch_size, embed_dim, conv_left_pad)
            name = f"cached_conv1_{i}"
            encoder_input[name] = tensors[4]
            encoder_output.append(f"new_{name}")

            # (batch_size, embed_dim, conv_left_pad)
            name = f"cached_conv2_{i}"
            encoder_input[name] = tensors[5]
            encoder_output.append(f"new_{name}")

        for i in range(len(self.states[:-2]) // 6):
            build_inputs_outputs(self.states[i * 6 : (i + 1) * 6], i)

        # (batch_size, channels, left_pad, freq)
        name = "embed_states"
        embed_states = self.states[-2]
        encoder_input[name] = embed_states
        encoder_output.append(f"new_{name}")

        # (batch_size,)
        name = "processed_lens"
        processed_lens = self.states[-1]
        encoder_input[name] = processed_lens
        encoder_output.append(f"new_{name}")

        return encoder_input, encoder_output

    def _update_states(self, states: List[np.ndarray]):
        self.states = states

    def run_encoder(self, x: torch.Tensor) -> torch.Tensor:
        """
        Args:
          x:
            A 3-D tensor of shape (N, T, C)
        Returns:
          Return a 3-D tensor of shape (N, T', joiner_dim) where
          T' is usually equal to ((T-7)//2-3)//2
        """
        encoder_input, encoder_output_names = self._build_encoder_input_output(x)

        out = self.encoder.run(encoder_output_names, encoder_input)

        self._update_states(out[1:])

        return torch.from_numpy(out[0])

    def run_decoder(self, decoder_input: torch.Tensor) -> torch.Tensor:
        """
        Args:
          decoder_input:
            A 2-D tensor of shape (N, context_size)
        Returns:
          Return a 2-D tensor of shape (N, joiner_dim)
        """
        out = self.decoder.run(
            [self.decoder.get_outputs()[0].name],
            {self.decoder.get_inputs()[0].name: decoder_input.numpy()},
        )[0]

        return torch.from_numpy(out)

    def run_joiner(
        self, encoder_out: torch.Tensor, decoder_out: torch.Tensor
    ) -> torch.Tensor:
        """
        Args:
          encoder_out:
            A 2-D tensor of shape (N, joiner_dim)
          decoder_out:
            A 2-D tensor of shape (N, joiner_dim)
        Returns:
          Return a 2-D tensor of shape (N, vocab_size)
        """
        out = self.joiner.run(
            [self.joiner.get_outputs()[0].name],
            {
                self.joiner.get_inputs()[0].name: encoder_out.numpy(),
                self.joiner.get_inputs()[1].name: decoder_out.numpy(),
            },
        )[0]

        return torch.from_numpy(out)


def read_sound_files(
    filenames: List[str], expected_sample_rate: float
) -> List[torch.Tensor]:
    """Read a list of sound files into a list 1-D float32 torch tensors.
    Args:
      filenames:
        A list of sound filenames.
      expected_sample_rate:
        The expected sample rate of the sound files.
    Returns:
      Return a list of 1-D float32 torch tensors.
    """
    ans = []
    for f in filenames:
        wave, sample_rate = torchaudio.load(f)
        assert (
            sample_rate == expected_sample_rate
        ), f"expected sample rate: {expected_sample_rate}. Given: {sample_rate}"
        # We use only the first channel
        ans.append(wave[0].contiguous())
    return ans


def create_streaming_feature_extractor() -> OnlineFeature:
    """Create a CPU streaming feature extractor.

    At present, we assume it returns a fbank feature extractor with
    fixed options. In the future, we will support passing in the options
    from outside.

    Returns:
      Return a CPU streaming feature extractor.
    """
    opts = FbankOptions()
    opts.device = "cpu"
    opts.frame_opts.dither = 0
    opts.frame_opts.snip_edges = False
    opts.frame_opts.samp_freq = 16000
    opts.mel_opts.num_bins = 80
    opts.mel_opts.high_freq = -400
    return OnlineFbank(opts)


def greedy_search(
    model: OnnxModel,
    encoder_out: torch.Tensor,
    context_size: int,
    decoder_out: Optional[torch.Tensor] = None,
    hyp: Optional[List[int]] = None,
) -> List[int]:
    """Greedy search in batch mode. It hardcodes --max-sym-per-frame=1.
    Args:
      model:
        The transducer model.
      encoder_out:
        A 3-D tensor of shape (1, T, joiner_dim)
      context_size:
        The context size of the decoder model.
      decoder_out:
        Optional. Decoder output of the previous chunk.
      hyp:
        Decoding results for previous chunks.
    Returns:
      Return the decoded results so far.
    """

    blank_id = 0

    if decoder_out is None:
        assert hyp is None, hyp
        hyp = [blank_id] * context_size
        decoder_input = torch.tensor([hyp], dtype=torch.int64)
        decoder_out = model.run_decoder(decoder_input)
    else:
        assert hyp is not None, hyp

    encoder_out = encoder_out.squeeze(0)
    T = encoder_out.size(0)
    for t in range(T):
        cur_encoder_out = encoder_out[t : t + 1]
        joiner_out = model.run_joiner(cur_encoder_out, decoder_out).squeeze(0)
        y = joiner_out.argmax(dim=0).item()
        if y != blank_id:
            hyp.append(y)
            decoder_input = hyp[-context_size:]
            decoder_input = torch.tensor([decoder_input], dtype=torch.int64)
            decoder_out = model.run_decoder(decoder_input)

    return hyp, decoder_out


@torch.no_grad()
def main():
    parser = get_parser()
    args = parser.parse_args()
    logging.info(vars(args))

    model = OnnxModel(
        encoder_model_filename=args.encoder_model_filename,
        decoder_model_filename=args.decoder_model_filename,
        joiner_model_filename=args.joiner_model_filename,
    )

    sample_rate = 16000

    logging.info("Constructing Fbank computer")
    online_fbank = create_streaming_feature_extractor()

    logging.info(f"Reading sound files: {args.sound_file}")
    waves = read_sound_files(
        filenames=[args.sound_file],
        expected_sample_rate=sample_rate,
    )[0]

    tail_padding = torch.zeros(int(0.3 * sample_rate), dtype=torch.float32)
    wave_samples = torch.cat([waves, tail_padding])

    num_processed_frames = 0
    segment = model.segment
    offset = model.offset

    context_size = model.context_size
    hyp = None
    decoder_out = None

    chunk = int(1 * sample_rate)  # 1 second
    start = 0
    while start < wave_samples.numel():
        end = min(start + chunk, wave_samples.numel())
        samples = wave_samples[start:end]
        start += chunk

        online_fbank.accept_waveform(
            sampling_rate=sample_rate,
            waveform=samples,
        )

        while online_fbank.num_frames_ready - num_processed_frames >= segment:
            frames = []
            for i in range(segment):
                frames.append(online_fbank.get_frame(num_processed_frames + i))
            num_processed_frames += offset
            frames = torch.cat(frames, dim=0)
            frames = frames.unsqueeze(0)
            encoder_out = model.run_encoder(frames)
            hyp, decoder_out = greedy_search(
                model,
                encoder_out,
                context_size,
                decoder_out,
                hyp,
            )

    token_table = k2.SymbolTable.from_file(args.tokens)

    text = ""
    for i in hyp[context_size:]:
        text += token_table[i]
    text = text.replace("▁", " ").strip()

    logging.info(args.sound_file)
    logging.info(text)

    logging.info("Decoding Done")


if __name__ == "__main__":
    formatter = "%(asctime)s %(levelname)s [%(filename)s:%(lineno)d] %(message)s"

    logging.basicConfig(format=formatter, level=logging.INFO)
    main()



# Copyright (c)  2022  Xiaomi Corporation (authors: Xiaoyu Yang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging

import torch

from icefall.checkpoint import average_checkpoints, load_checkpoint
from icefall.rnn_lm.model import RnnLmModel
from icefall.transformer_lm.model import TransformerLM
from icefall.utils import AttributeDict, str2bool


class LmScorer(torch.nn.Module):
    """This is a wrapper for NN LMs
    The language models supported include:
        RNN,
        Transformer
    """

    def __init__(
        self,
        lm_type: str,
        params: AttributeDict,
        device,
        lm_scale: float = 0.3,
    ):
        super(LmScorer, self).__init__()
        assert lm_type in ["rnn", "transformer"], f"{lm_type} is not supported"
        self.lm_type = lm_type
        self.lm = self.get_lm(lm_type, device, params)
        self.lm_scale = lm_scale
        self.params = params

    @classmethod
    def add_arguments(cls, parser):
        # LM general arguments
        parser.add_argument(
            "--lm-vocab-size",
            type=int,
            default=500,
        )

        parser.add_argument(
            "--lm-epoch",
            type=int,
            default=7,
            help="""Which epoch to be used
            """,
        )

        parser.add_argument(
            "--lm-avg",
            type=int,
            default=1,
            help="""Number of checkpoints to be averaged
            """,
        )

        parser.add_argument("--lm-exp-dir", type=str, help="Path to LM experiments")

        # Now RNNLM related arguments
        parser.add_argument(
            "--rnn-lm-embedding-dim",
            type=int,
            default=2048,
            help="Embedding dim of the model",
        )

        parser.add_argument(
            "--rnn-lm-hidden-dim",
            type=int,
            default=2048,
            help="Hidden dim of the model",
        )

        parser.add_argument(
            "--rnn-lm-num-layers",
            type=int,
            default=3,
            help="Number of RNN layers the model",
        )

        parser.add_argument(
            "--rnn-lm-tie-weights",
            type=str2bool,
            default=True,
            help="""True to share the weights between the input embedding layer and the
            last output linear layer
            """,
        )

        # Now transformers
        parser.add_argument(
            "--transformer-lm-exp-dir", type=str, help="Directory of transformer LM exp"
        )

        parser.add_argument(
            "--transformer-lm-dim-feedforward",
            type=int,
            default=2048,
            help="Dimension of FFW module in transformer",
        )

        parser.add_argument(
            "--transformer-lm-encoder-dim",
            type=int,
            default=768,
            help="Encoder dimension of transformer",
        )

        parser.add_argument(
            "--transformer-lm-embedding-dim",
            type=int,
            default=768,
            help="Input embedding dimension of transformer",
        )

        parser.add_argument(
            "--transformer-lm-nhead",
            type=int,
            default=8,
            help="Number of attention heads in transformer",
        )

        parser.add_argument(
            "--transformer-lm-num-layers",
            type=int,
            default=16,
            help="Number of encoder layers in transformer",
        )

        parser.add_argument(
            "--transformer-lm-tie-weights",
            type=str2bool,
            default=True,
            help="If tie weights in transformer LM",
        )

    def get_lm(self, lm_type: str, device, params: AttributeDict) -> torch.nn.Module:
        """Return the neural network LM

        Args:
            lm_type (str): Type name of NN LM
        """
        if lm_type == "rnn":
            model = RnnLmModel(
                vocab_size=params.lm_vocab_size,
                embedding_dim=params.rnn_lm_embedding_dim,
                hidden_dim=params.rnn_lm_hidden_dim,
                num_layers=params.rnn_lm_num_layers,
                tie_weights=params.rnn_lm_tie_weights,
            )

            if params.lm_avg == 1:
                load_checkpoint(
                    f"{params.lm_exp_dir}/epoch-{params.lm_epoch}.pt", model
                )
                model.to(device)
            else:
                start = params.lm_epoch - params.lm_avg + 1
                filenames = []
                for i in range(start, params.lm_epoch + 1):
                    if start >= 0:
                        filenames.append(f"{params.lm_exp_dir}/epoch-{i}.pt")
                logging.info(f"averaging {filenames}")
                model.to(device)
                model.load_state_dict(average_checkpoints(filenames, device=device))

        elif lm_type == "transformer":
            model = TransformerLM(
                vocab_size=params.lm_vocab_size,
                d_model=params.transformer_lm_encoder_dim,
                embedding_dim=params.transformer_lm_embedding_dim,
                dim_feedforward=params.transformer_lm_dim_feedforward,
                nhead=params.transformer_lm_nhead,
                num_layers=params.transformer_lm_num_layers,
                tie_weights=params.transformer_lm_tie_weights,
                params=params,
            )

            if params.lm_avg == 1:
                load_checkpoint(
                    f"{params.lm_exp_dir}/epoch-{params.lm_epoch}.pt", model
                )
                model.to(device)
            else:
                start = params.lm_epoch - params.lm_avg + 1
                filenames = []
                for i in range(start, params.lm_epoch + 1):
                    if start >= 0:
                        filenames.append(f"{params.lm_exp_dir}/epoch-{i}.pt")
                logging.info(f"averaging {filenames}")
                model.to(device)
                model.load_state_dict(average_checkpoints(filenames, device=device))
        else:
            raise NotImplementedError()

        return model

    def score_token(self, x: torch.Tensor, x_lens: torch.Tensor, state=None):
        """Score the input and return the prediction
        This requires the lm to have the method `score_token`
        Args:
            x (torch.Tensor): Input tokens
            x_lens (torch.Tensor): Length of the input tokens
            state (optional): LM states

        """
        return self.lm.score_token(x, x_lens, state)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    LmScorer.add_arguments(parser)
    args = parser.parse_args()

    params = AttributeDict()
    params.update(vars(args))

    device = torch.device("cpu")
    if torch.cuda.is_available():
        device = torch.device("cuda", 0)

    Scorer = LmScorer(params=params, device=device)
    Scorer.eval()

    x = (
        torch.tensor([[1, 4, 19, 256, 77], [1, 4, 19, 256, 77]])
        .to(device)
        .to(torch.int64)
    )
    x_lens = torch.tensor([5, 5]).to(device)

    state = None

    score, state = Scorer.score(x, x_lens)
    print(score.shape)
    print(score[0])
    print(score[1])



from gxl_ai_utils.utils import utils_file
# path_1 = "/mnt/sfs/asr/update_data/3500_chat_asr/3500_with_asr_chat.list"
# output_path =  "/mnt/sfs/asr/update_data/3500_chat_asr/3500_with_asr_chat.jsonl"
# lines = utils_file.load_list_file_clean(path_1)
# dict_list = []
# for line in lines:
#     dict_i = {}
#     items = line.strip().split('\t')
#     if len(items) != 5:
#         utils_file.logging_warning("line error: {} len != 3 , continue".format(line) )
#         continue
#     dict_i["key"] = items[0]
#     dict_i["Q"] = items[2]
#     dict_i["A"] = items[3]
#     dict_list.append(dict_i)
# utils_file.write_dict_list_to_jsonl(dict_list, output_path)

root_dir = "/mnt/sfs/asr/update_data/3500_chat_asr"
path_list = utils_file.do_get_list_for_wav_dir(root_dir, suffix='jsonl')
res_dict = {}
for path_i in path_list:
    dict_list = utils_file.load_dict_list_from_jsonl(path_i)
    for dict_i in utils_file.tqdm(dict_list, total=len(dict_list)):
        key = dict_i["key"]
        if key in res_dict:
            utils_file.logging_warning("key {} exist in res_dict".format(key))
        new_txt = dict_i["Q"].replace(" ", "") + "<回答>" + dict_i["A"].replace(" ", "")
        res_dict[key] = new_txt
output_path = "/mnt/sfs/asr/update_data/3500_chat_asr/gxl_all_3500_with_asr_chat.scp"
utils_file.write_dict_to_scp(res_dict, output_path)



import torch
from torch import nn
from torchvision import models
from torchvision.models import ResNet152_Weights, ResNet18_Weights

from gxl_ai_utils import utils

from .rnn_model import *
from .cnn_model import *
from .attention_model import * # 这三个文件中的类对象均可以通过get_model_by_name直接由字符串得到
from .mlp_model import *

class MnistModel0(nn.Module):
    def __init__(self, ) -> None:
        super().__init__()
        self.conv1 = nn.Conv2d(in_channels=1, out_channels=32, kernel_size=3)  # 28->26
        self.conv2 = nn.Conv2d(in_channels=32, out_channels=64, kernel_size=3)  # 26->24
        self.pool = nn.MaxPool2d(kernel_size=2)  # ->减半
        self.fc1 = nn.Linear(64 * 5 * 5, 128)
        self.fc2 = nn.Linear(128, 10)  # 10 classes for MNIST

    def forward(self, x):
        x = self.conv1(x)
        x = torch.relu(x)
        x = self.pool(x)
        x = self.conv2(x)
        x = torch.relu(x)
        x = self.pool(x)
        x = x.view(-1, 64 * 5 * 5)
        x = self.fc1(x)
        x = torch.relu(x)
        x = self.fc2(x)
        return torch.nn.functional.softmax(x, dim=1)


class ResNet152FlowerModel(nn.Module):
    def __init__(self, output_num=102, feature_extract=True):
        self.resnet_block = models.resnet152(weights=ResNet152_Weights.DEFAULT)
        utils.utils_model.set_parameter_requires_grad(self.resnet_block, feature_extract)
        fc_in = self.resnet_block.fc.in_features
        self.resnet_block.fc = nn.Linear(fc_in, output_num)

    def forward(self, X):
        return self.resnet_block(X)


class ResNet18FlowerModel(nn.Module):
    def __init__(self, output_num=102, feature_extract=True):
        super(ResNet18FlowerModel, self).__init__()
        self.resnet_block = models.resnet18(weights=ResNet18_Weights.DEFAULT)
        utils.utils_model.set_parameter_requires_grad(self.resnet_block, feature_extract)
        fc_in = self.resnet_block.fc.in_features
        self.resnet_block.fc = nn.Linear(fc_in, output_num)

    def forward(self, X):
        return self.resnet_block(X)


def get_model_by_name(name, **kwargs):
    b = utils.utils_model.create_instance_by_name("gxl_ai_utils.store_model.store_model_class.store_model_class", name, **kwargs)
    return b.to(torch.device('cpu'))



from matplotlib import pyplot as plt

from gxl_ai_utils.utils import utils_file
import os


def get_data_from_first_dir(input_dir_path, res_dict):
    dataset_name_list = os.listdir(input_dir_path)
    for dataset_name in dataset_name_list:
        temp_wer_path = os.path.join(input_dir_path, dataset_name, "wer")
        if not os.path.exists(temp_wer_path):
            utils_file.logging_print(f'{temp_wer_path} not exists, not handle it')
            continue
        if not "speechio" in dataset_name:
            utils_file.logging_print(f'{dataset_name} not speechio, not handle it')
            continue
        wer_num = utils_file.get_wer_all_from_wer_file(temp_wer_path)
        res_dict[dataset_name] = wer_num
    print(res_dict)
    print(list(res_dict.values()))

if __name__ == '__main__':
    info_dict = utils_file.load_data_from_xlsx("/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/prepare_data_for_salmonn/vision_4_prompt_task/tongyi_fenxi/down2_stage3__version2/data/wer_paths2.xlsx", return_cols=False)
    # for common
    common_list = []
    for i in range(27):
        key = f"speechio_{i}"
        common_wer_path = info_dict[key][0]
        print(common_wer_path)
        res_wer = utils_file.get_wer_all_from_wer_file(common_wer_path)
        print(res_wer)
        replace_num = int(res_wer[1])
        common_list.append(replace_num)
    short_list = []
    for i in range(27):
        key = f"speechio_{i}"
        common_wer_path = info_dict[key][1]
        print(common_wer_path)
        res_wer = utils_file.get_wer_all_from_wer_file(common_wer_path)
        print(res_wer)
        replace_num = int(res_wer[1])
        short_list.append(replace_num)
    long_list = []
    for i in range(27):
        key = f"speechio_{i}"
        common_wer_path = info_dict[key][2]
        print(common_wer_path)
        res_wer = utils_file.get_wer_all_from_wer_file(common_wer_path)
        print(res_wer)
        replace_num = int(res_wer[1])
        long_list.append(replace_num)

    # 得到提升的百分比
    messy_percent_list = [(common_list[i] - item) / common_list[i] * 100 for i, item in enumerate(long_list)]
    short_messy_list = [(common_list[i] - item) / common_list[i] * 100 for i, item in enumerate(short_list)]
    common_messy_list = [(common_list[i] - item) / common_list[i] * 100 for i, item in enumerate(common_list)]
    # X轴的值
    x_values = list(range(27))

    # 创建折线图
    plt.figure(figsize=(10, 6))
    plt.plot(x_values, messy_percent_list, label='long-domain', marker='o')
    plt.plot(x_values, short_messy_list, label='short-domain', marker='s')
    plt.plot(x_values, common_messy_list, label='common', marker='^')

    # 添加标题和标签
    # plt.title('Performance improvement percentage of domain prompts compared to common prompt in substitution errors.')
    plt.xlabel('Index (speechio_*)')
    plt.ylabel('Value (%)')

    # 设置X轴刻度
    plt.xticks(x_values)

    # 添加图例
    plt.legend()

    # 显示图形
    plt.grid()
    # plt.show()
    plt.savefig('./table3.png')


def get_all_parttern_from_root_dir(root_dir,pt_name, down_num=4, stage=2):
    """"""
    # root_dir = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/exp/salmonn_v16/5_epoch"
    dir_path_list, _ = utils_file.do_listdir(root_dir)
    pattern_list = [
        "common",
        "short",
        "long",
        "repeat",
        "encourage",
        "messy"
    ]
    dataset_list = [f"speechio_{i}" for i in range(27)]
    final_res_dict = {"类型/数据集": dataset_list}
    for pattern in pattern_list:
        print(pattern)
        res_dict4pattern = {}
        for dir_path in dir_path_list:
            print(dir_path)
            if not pt_name in dir_path:
                utils_file.logging_print(f'{dir_path} not {pt_name}, not handle it')
                continue
            if pattern in dir_path:
                get_data_from_first_dir(dir_path, res_dict4pattern)
            else:
                utils_file.logging_print(f'{pattern} not {dir_path}, not handle it')
                continue
        res_list = []
        for dataset in dataset_list:
            if dataset in res_dict4pattern:
                res_list.append(res_dict4pattern[dataset])
            else:
                res_list.append(-1)
        final_res_dict[pattern] = res_list
    utils_file.write_dict_to_json(final_res_dict, f"./xlsx_data/down_{down_num}_stage_{stage}.json")
    utils_file.write_dict_to_xlsx(final_res_dict, f"./xlsx_data/down_{down_num}_stage_{stage}.xlsx")

#
# if __name__ == '__main__':
#     root_dir = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/exp/salmonn_v16/5_epoch"
#     down_num = 4
#     stage_num = 3
#     pt_name= "epoch_0"
#     root_dir = "/home/work_nfs8/xlgeng/new_workspace/wenet_gxl_salmonn/examples/aishell/salmonn/exp/salmonn_v16/3_epoch/test_step_29988/speechio"
#     down_num = 4
#     stage_num = 2
#     pt_name = "step_29988"
#     get_all_parttern_from_root_dir(root_dir,pt_name, down_num, stage_num)




from __future__ import absolute_import, division, print_function, unicode_literals

import glob
import os
import numpy as np
import argparse
import json
import torch
from tqdm import tqdm
from time import time
from utils import AttrDict
from models import Generator
import soundfile as sf
from scipy.interpolate import interp1d

h = None
device = None


def load_checkpoint(filepath, device):
    assert os.path.isfile(filepath)
    print("Loading '{}'".format(filepath))
    checkpoint_dict = torch.load(filepath, map_location=device)
    print("Complete.")
    return checkpoint_dict


def scan_checkpoint(cp_dir, prefix):
    pattern = os.path.join(cp_dir, prefix + '*')
    cp_list = glob.glob(pattern)
    if len(cp_list) == 0:
        return ''
    return sorted(cp_list)[-1]


def process_data(wavlm_file):
    wavlm = np.load(wavlm_file)
    return torch.from_numpy(wavlm).unsqueeze(0)


def inference(a):
    generator = Generator(h).to(device)

    state_dict_g = load_checkpoint(a.checkpoint_file, device)
    generator.load_state_dict(state_dict_g['generator'])

    filelist = os.listdir(a.input)
    os.makedirs(a.output, exist_ok=True)

    generator.eval()
    generator.remove_weight_norm()
    rtfs = []


    print(generator)
    # exit()
    with torch.no_grad():
        for i, filename in tqdm(enumerate(filelist)):
            start = time()
            wavlm_feature = process_data(os.path.join(a.input, filename))
            wavlm_feature = wavlm_feature.to(device)
            y_g_hat = generator(wavlm_feature)
            audio = y_g_hat.squeeze()
            rtf = (time() - start) / (audio.size(-1) / h.sampling_rate)
            rtfs.append(rtf)
            audio = audio.cpu().numpy()

            output_file = os.path.join(a.output, os.path.splitext(filename)[0] + '.wav')
            sf.write(output_file, audio, h.sampling_rate)
            print(output_file, f"\tRTF: {rtf:.4f}")
    print(f"Avg RTF: {sum(rtfs) / len(rtfs):.6f}")


def main():
    print('Initializing Inference Process..')

    parser = argparse.ArgumentParser()
    parser.add_argument('--input', default='test_mel_files')
    parser.add_argument('--output', default='generated_files_from_mel')
    parser.add_argument('--checkpoint_file',
                        default="/home/work_nfs7/ypjiang/code/knn-hifigan/logs/wavlm_24k/g_00480000.pt")
    a = parser.parse_args()

    config_file = os.path.join(os.path.split(a.checkpoint_file)[0], 'config.json')
    with open(config_file) as f:
        data = f.read()

    global h
    json_config = json.loads(data)
    h = AttrDict(json_config)

    torch.manual_seed(h.seed)
    global device
    if torch.cuda.is_available():
        torch.cuda.manual_seed(h.seed)
        device = torch.device('cuda')
    else:
        device = torch.device('cpu')

    inference(a)


if __name__ == '__main__':
    main()




import argparse
import os.path


from gxl_ai_utils.utils import utils_file
import re

def get_tag_from_str(input_str):
    # 使用正则表达式提取所有 <> 中的内容
    matches = re.findall(r'<.*?>', input_str)
    # 输出结果
    if len(matches)==0:
        return '<--no_tag-->'
    return matches[0].upper()

def convert_to_nearest_multiple(numbers, step=0.3):
    # 使用numpy的四舍五入功能将每个数字转换为最近的step的倍数
    return [round(num / step) * step for num in numbers]

def get_full_tag_from_str(input_str, smooth_interval=0.1):
    matches = re.findall(r'<.*?>', input_str)
    # 输出结果
    # if len(matches) == 0:
    #     return '<--no_tag-->'
    num_list = []
    for match in matches:
        try:
            # 尝试将匹配到的字符串转换为浮动数
            num_list.append(float(match[1:-1]))  # 去掉字符串的首尾字符后转换为 float
        except ValueError:
            # 如果转换失败，则将 -1 添加到 num_list
            num_list.append(-1)
    num_list = convert_to_nearest_multiple(num_list, step=smooth_interval)
    res_str = " ".join([f"{num:.6f}" for num in num_list])
    return res_str

def do_replace_tag(input_file, output_file):
    # 打开文件A并读取内容
    with open(input_file, 'r', encoding='utf-8') as file_a:
        content = file_a.read()

    # 删除文件内容中的 < 和 >
    content = content.replace('<', '').replace('>', '')
    # 将修改后的内容写入文件B
    with open(output_file, 'w', encoding='utf-8') as file_b:
        file_b.write(content)

def do_convert_to_pure_tag(input_file, output_file):
    text_dict = utils_file.load_dict_from_scp(input_file)
    new_text_dict = {}
    num_blank_tag = 0
    for key , value in text_dict.items():
        res_value = get_full_tag_from_str(value, smooth_interval=0.2)
        new_text_dict[key] =res_value
        if len(res_value) == 0:
            num_blank_tag += 1
    utils_file.logging_warning('空白时间戳的数量为：', num_blank_tag)
    output_file_blank_num = output_file + '_blank_num'
    utils_file.write_list_to_file([f'空白帧数量为{num_blank_tag}'],output_file_blank_num)
    utils_file.write_dict_to_scp(new_text_dict, output_file)

if __name__ == '__main__':
    """
    首先对识别结果进行计算，接着计算acc,
    分别存入wer文件和acc文件
    """
    parser = argparse.ArgumentParser()
    task = 'align'
    parser.add_argument('--ref_file', type=str, required=True, help='reference file, 真实的标签')
    parser.add_argument('--hyp_file', type=str, required=True, help='hypothesis file， 推理的标签')
    parser.add_argument('--output_dir', type=str, required=True, help='output dir 结果保存的目录')
    args = parser.parse_args()
    utils_file.makedir_sil(args.output_dir)
    utils_file.logging_info(f'开始计算{task}的wer')
    utils_file.do_compute_wer(args.ref_file, args.hyp_file, args.output_dir)
    utils_file.logging_info('字符错误率计算完毕，结果如下')
    res = utils_file.do_execute_shell_command(f'tail -n 8 {os.path.join(args.output_dir, "wer")}')
    print(res)

    utils_file.logging_info(f'开始计算{task}纯标签的wer_smooth')
    acc_path = os.path.join(args.output_dir, 'wer_pure_tag_smooth')
    hyp_dict = utils_file.load_dict_from_scp(args.hyp_file)
    ref_dict = utils_file.load_dict_from_scp(args.ref_file)
    hyp_file_tmp = utils_file.do_get_fake_file()
    ref_file_tmp = utils_file.do_get_fake_file()
    do_convert_to_pure_tag(args.hyp_file, hyp_file_tmp)
    do_convert_to_pure_tag(args.ref_file, ref_file_tmp)
    output_dir_tmp = '/home/xlgeng/.cache/'
    utils_file.do_compute_wer(ref_file_tmp, hyp_file_tmp, output_dir_tmp)
    utils_file.copy_file(os.path.join(output_dir_tmp, 'wer'), acc_path)
    utils_file.logging_info('计算{task}纯标签的wer_smooth计算完毕，结果如下')
    res = utils_file.do_execute_shell_command(f'tail -n 12 {acc_path}')
    print(res)

    utils_file.logging_info('计算空时间戳的数量')














import os
import sys
sys.path.insert(0, '../../../../../')
from gxl_ai_utils.utils import utils_file


test_sets_list = [
    "style",    "chat",     "add_mid_background",   "pure_background",  "add_end_background",   "rupt_data_mid",
    "emotion",  "align",    "gender",               "age",              "caption",              "rupt_data_end",
]
# test_sets_list = [
#     "meld",
# ]
test_sets_list = [
    "public_test/roobo_100",
    "caption","caption_0107_esc50","caption_0107_vocalsound","caption_aslp_record",
    "emotion", "public_test/MELD_test", "public_test/MER23_test",
    "style",
    "gender", "public_test/aishell1_gender", "public_test/kaggle_gender",
    "age", "public_test/kaggle_age",
    "chat", "public_test/AirBench_speech",
    # asr
]
# test_sets_list = []
asr_sets_list = [
    "aishell2", "librispeech_clean", "librispeech_other", "test_net_1", "test_net_2", "test_meeting",
    "speechio_0", "speechio_1", "speechio_2", "speechio_3", "speechio_4",
]

dirs=[
    "/mnt/sfs/asr/ckpt/qwen2_multi_task_4_6gpus_gxl_adapter/epoch_5_with_speech_gxl",
    # "/mnt/sfs/asr/ckpt/qwen2_multi_task_4_6gpus_gxl_adapter/epoch_2_with_speech_gxl",
    # "/mnt/sfs/asr/ckpt/qwen2_multi_task_4_6gpus_gxl_adapter/epoch_1_with_speech_gxl",
]

# 定义不同的检查点名称列表，与dirs列表中的元素顺序对应，可根据实际调整对应关系
ckpt_names=[
    "step_21249.pt",
    # "step_28999.pt",
    # "step_18999.pt",
]

import gxl_utils
# test_data_dir='/home/node54_tmpdata/xlgeng/code/wenet_whisper_finetune_xlgeng/examples/wenetspeech/whisper/data/test_sets_format_3000'
test_data_dir = "/mnt/sfs/asr/test_data/test_sets_format_3000"
asr_test_dir = "/mnt/sfs/asr/test_data/asr_test_sets"
res_dict = {}
index = -1
for dir_i, ckpt_name in zip(dirs, ckpt_names):
    index += 1
    res_dir_i = f"{dir_i}/test_{ckpt_name}"
    utils_file.logging_info(f'开始计算 {res_dir_i} 的结果')
    big_res_dict = {}
    for test_set in test_sets_list:
        try:
            utils_file.logging_info(f'test_set {test_set}')
            text_res_path = f'{res_dir_i}/{test_set}/llmasr_decode/text'
            ref_text_path = f'{test_data_dir}/{test_set}/text'
            output_dir1 = f'{res_dir_i}/{test_set}'
            output_dir2 = f'./exp/{index}_{ckpt_name.replace(".pt", "")}/{test_set}'
            if not utils_file.if_file_exist(text_res_path):
                utils_file.logging_info(f'{text_res_path} 不存在，跳过')
                continue
            else:
                row_num = utils_file.do_get_file_rows_num_shell(text_res_path)
                utils_file.logging_info(f'{text_res_path} 有 {row_num} 行')
            if test_set == 'chat' or test_set == "public_test/AirBench_speech":
                res_dict = gxl_utils.do_compute_bleu_for_chat(ref_text_path, text_res_path, output_dir1, output_dir2)
            elif test_set == 'align' or test_set== "public_test/roobo_100":
                res_dict = gxl_utils.do_compute_align2(ref_text_path, text_res_path, output_dir1, output_dir2)
            else:
                res_dict = gxl_utils.do_compute_acc(ref_text_path, text_res_path, output_dir1, output_dir2)
            utils_file.logging_warning(f'RES: {test_set} {res_dict}')
            big_res_dict[test_set] = res_dict
        except Exception as e:
            utils_file.logging_error(f'计算 {test_set} 的结果出错，错误信息为 {e}')
    for test_set in asr_sets_list:
        try:
            res_dict = {}
            utils_file.logging_info(f'test_set {test_set}')
            text_res_path = f'{res_dir_i}/{test_set}/llmasr_decode/wer'
            rext_res_tmp_path = "/home/xlgeng/wer"
            ref_text_path = f'{asr_test_dir}/{test_set}/text'
            hyp_test_path =f'{res_dir_i}/{test_set}/llmasr_decode/text'
            if not utils_file.if_file_exist(hyp_test_path):
                utils_file.logging_info(f'{hyp_test_path} 不存在')
                continue
            if not utils_file.if_file_exist(text_res_path):
                utils_file.logging_info(f'{text_res_path} 不存在')
                res_dict['infering'] = 'true'
                utils_file.do_compute_wer(ref_text_path, hyp_test_path, "/home/xlgeng")
                wer_num = utils_file.do_get_wer_from_wer_file4all(rext_res_tmp_path)
                wer_chinese = utils_file.do_get_wer_from_wer_file4mandarin(rext_res_tmp_path)
                wer_english = utils_file.do_get_wer_from_wer_file4english(rext_res_tmp_path)
            else:
                res_dict['infering'] = 'false'
                wer_num = utils_file.do_get_wer_from_wer_file4all(text_res_path)
                wer_chinese = utils_file.do_get_wer_from_wer_file4mandarin(text_res_path)
                wer_english = utils_file.do_get_wer_from_wer_file4english(text_res_path)
                if wer_chinese == -1 and wer_english == -1:
                    utils_file.logging_info(f'{test_set} 的wer为0')
                    res_dict['infering'] = 'true'
                    utils_file.do_compute_wer(ref_text_path, hyp_test_path, f'/home/xlgeng')
                    wer_num = utils_file.do_get_wer_from_wer_file4all(rext_res_tmp_path)
                    wer_chinese = utils_file.do_get_wer_from_wer_file4mandarin(rext_res_tmp_path)
                    wer_english = utils_file.do_get_wer_from_wer_file4english(rext_res_tmp_path)


            res_dict['wer_all'] = wer_num
            res_dict['wer_chinese'] = wer_chinese
            res_dict['wer_english'] = wer_english
            big_res_dict[test_set] = res_dict

        except Exception as e:
            utils_file.logging_error(f'计算 {test_set} 的结果出错，错误信息为 {e}')
    utils_file.write_dict_to_scp(big_res_dict, os.path.join(f'./exp/{ckpt_name.replace(".pt", "")}', 'big_res.scp'))






"""
从caption的测试集的data.list得到text
"""
from gxl_ai_utils.utils import utils_file

input_dir = "/home/node54_tmpdata/xlgeng/code/wenet_whisper_finetune/examples/wenetspeech/whisper/gxl_data/test_sets/caption"
partions = ['caption_1','caption_2']
for partion in partions:
    tmp_data_list = utils_file.join_path(input_dir, partion, 'data.list')
    output_text_path = utils_file.join_path(input_dir, partion, 'text')
    dict_list = utils_file.load_dict_list_from_jsonl(tmp_data_list)
    text_dict = {}
    for dict_i in dict_list:
        text_dict[dict_i['key']] = dict_i['txt']
    utils_file.write_dict_to_scp(text_dict, output_text_path)



from gxl_ai_utils.utils import utils_file

input_list_path = "/home/work_nfs8/xlgeng/new_workspace/gxl_ai_utils/eggs/cats_and_dogs/prepare_data_for_salmonn_TTS/wenetspeech_acoustic/wtn_train.list"
utils_file.do_make_shard_file("")


import numpy

from gxl_ai_utils.utils import utils_file

if __name__ == '__main__':
    str_list = utils_file.load_list_file_clean(
        'E:\gengxuelong_study\server_local_adapter\gxl_work\gxl_ai_utils\eggs\knowledge_distillation\output\paraformer\speech_paraformer-large_asr_nat-zh-cn-16k-common-vocab8404-pytorch\\am.mvn')
    # utils_file.print_list(str_list)
    res_list = []
    for i, str in enumerate(str_list):
        if i == 4 or i == 6:
            res_list.append(str.split('<LearnRateCoef> 0')[1].strip())
    utils_file.print_list(res_list)
    mean_list_str = res_list[0]
    std_list_str = res_list[1]
    mean_list = [float(i) for i in mean_list_str[1:len(mean_list_str) - 1].split()]
    std_list = [float(i) for i in std_list_str[1:len(std_list_str) - 1].split()]
    print(mean_list)
    print(std_list)
    mean = numpy.array(mean_list)
    std = numpy.array(std_list)
    mean = -1 * mean
    std = 1.0 / std
    print(mean)
    print(std)
    dic = dict(dim=560, mean=mean, std=std)
    utils_file.write_dict_to_json(dic, './mvn.json')



import re

from gxl_ai_utils.utils import utils_file

import argparse
import os

def parse_args():
    parser = argparse.ArgumentParser(description='Compute accuracy')
    parser.add_argument('--ref_path', type=str, default='config.yaml', help='config file path')
    parser.add_argument('--hyp_path', type=str, default='model.pth', help='model file path')
    parser.add_argument('--output_path', type=str, default='data', help='data file path')

    args = parser.parse_args()
    return args

def do_get_first_tag_from_str(input_str, upper=True):
    """
    得到第一个<>标签的内容，并将其转为大写
    >>> input : "<tag1>content1<tag2>content2<tag3>content3"
    >>> output: "<TAG1>"
    :param upper: if false, return the lower case of the tag
    :param input_str:
    :return:
    """
    # 使用正则表达式提取所有 <> 中的内容
    matches = re.findall(r'<.*?>', input_str)
    # 输出结果
    if len(matches)==0:
        return '<--no_tag-->'
    if upper:
        return matches[0].upper()
    else:
        return matches[0].lower()

def do_showing_confusion_matrix(labels, matrix,title='', output_fig_path: str=None):
    """
    可视化混淆矩阵的函数
    :param labels: 标签序列，类型为列表等可迭代对象
    :param matrix: 代表混淆矩阵的二维列表，元素为整数，形状应为(len(labels), len(labels))
    """
    import matplotlib.pyplot as plt
    import numpy as np
    # 设置中文字体为黑体，以支持中文显示（确保系统中已安装黑体字体）
    plt.rcParams['font.sans-serif'] = ['SimHei']
    # 解决负号显示问题（有时候会出现负号显示异常，这一步是为了保证显示正常）
    plt.rcParams['axes.unicode_minus'] = False
    num_classes = len(labels)
    fig, ax = plt.subplots()
    # 使用imshow来绘制热力图展示混淆矩阵
    im = ax.imshow(np.array(matrix), cmap=plt.cm.Blues)

    # 设置x轴和y轴的刻度以及对应的标签
    ax.set_xticks(np.arange(num_classes))
    ax.set_yticks(np.arange(num_classes))
    ax.set_xticklabels(labels)
    ax.set_yticklabels(labels)

    # 旋转x轴刻度标签，让其更美观显示
    plt.setp(ax.get_xticklabels(), rotation=45, ha="right",
             rotation_mode="anchor")

    # 添加每个方格中的数值标签
    for i in range(num_classes):
        for j in range(num_classes):
            text = ax.text(j, i, matrix[i][j],
                           ha="center", va="center", color="black")

    ax.set_title(f"{title} Confusion Matrix")
    ax.set_xlabel('Predicted label')
    ax.set_ylabel('True label')
    fig.tight_layout()
    if output_fig_path is not None:
        plt.savefig(output_fig_path)
    else:
        plt.show()


def do_compute_acc(ref_file, hyp_file, output_path):
    """
    计算正确率和错误率，并列出混淆矩阵
    :return:
    """
    output_dir = os.path.dirname(output_path)
    utils_file.makedir_sil(output_dir)
    utils_file.logging_info('开始计算正确率，acc')
    acc_path = output_path
    hyp_dict = utils_file.load_dict_from_scp(hyp_file)
    ref_dict = utils_file.load_dict_from_scp(ref_file)
    tag_hyp_dict = {}
    for key, value in hyp_dict.items():
        tag_hyp_dict[key] = do_get_first_tag_from_str(value)
    tag_ref_dict = {}
    for key, value in ref_dict.items():
        tag_ref_dict[key] = do_get_first_tag_from_str(value)
    output_acc_f = open(acc_path, 'w', encoding='utf-8')
    same_num = 0
    all_tags = set()
    # utils_file.print_dict(utils_file.do_get_random_subdict(tag_ref_dict, 10))
    for tag in tag_ref_dict.values():
        all_tags.add(tag)
    labels = sorted(list(all_tags))
    utils_file.logging_info(f'标签种类为：{labels}')
    num_classes = len(labels)
    matrix = [[0] * num_classes for _ in range(num_classes)]
    for key, hyp_tags in tag_hyp_dict.items():
        if key not in tag_ref_dict:
            continue
        ref_tags = tag_ref_dict[key]
        # 判断标签是否相同
        if ref_tags == hyp_tags:
            if_same = True
            same_num += 1
            index = labels.index(ref_tags)
            matrix[index][index] += 1
        else:
            if_same = False
            # 标签不同的情况，找到真实标签和预测标签对应的索引，在相应的位置加1
            ref_index = labels.index(ref_tags)
            if hyp_tags in labels:
                hyp_index = labels.index(hyp_tags)
                matrix[ref_index][hyp_index] += 1
        # 向文件写入数据
        output_acc_f.write(f"key: {key}\n")
        output_acc_f.write(f"ref_tag: {ref_tags}\n")
        output_acc_f.write(f"hyp_tag: {hyp_tags}\n")
        output_acc_f.write(f"if_same: {if_same}\n")
        output_acc_f.write("\n")  # 添加空行分隔不同的条目
    acc_num = same_num / len(tag_hyp_dict)
    output_acc_f.write(f'正确率为：{acc_num}')
    output_acc_f.flush()
    output_acc_f.close()
    figure_path = os.path.join(output_dir, 'confusion_matrix.png')
    do_showing_confusion_matrix(labels, matrix, output_fig_path=figure_path)
    return {
        'acc': acc_num,
    }

if __name__ == '__main__':
    args = parse_args()
    ref_file = args.ref_path
    hyp_file = args.hyp_path
    output_path = args.output_path
    do_compute_acc(ref_file, hyp_file, output_path)


from gxl_ai_utils.utils import utils_file

meld_path = "/mnt/sfs/asr/test_data/test_sets_format_3000/public_test/MELD_test"

mer_path = "/mnt/sfs/asr/test_data/test_sets_format_3000/public_test/MER23_test"

# for meld
dict_list = utils_file.load_dict_list_from_jsonl(f'{meld_path}/data.list')
text_dict = {}
for dict_i in dict_list:
    text_dict[dict_i["key"]] = dict_i["txt"] + "<" + dict_i["emotion"] +">"
utils_file.write_dict_to_scp(text_dict,
                            f"{meld_path}/text")
# for mer
dict_list = utils_file.load_dict_list_from_jsonl(f'{mer_path}/data.list')
text_dict = {}
for dict_i in dict_list:
    text_dict[dict_i["key"]] = "<" + dict_i["emotion"] +">"
utils_file.write_dict_to_scp(text_dict,
                            f"{mer_path}/text")


from gxl_ai_utils.utils import utils_file

def test_do_get_row_scp():
    """"""
    wav_dir = "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/data/wav_row"
    wav_scp_path = "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/data/scp/wav.scp"
    wav_dict = utils_file.do_get_scp_for_wav_dir(wav_dir)
    utils_file.write_dict_to_scp(wav_dict, wav_scp_path)
    text_scp_path = "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/data/scp/text"
    text_dict = {}
    for key, wav_path in  wav_dict.items():
        text_dict[key] = "我是耿雪龙"
    utils_file.write_dict_to_scp(text_dict, text_scp_path)

def test_do_get_fbank_k2():
    """"""
    fbank_dir = "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/data/k2/fbank"
    manifest_dir = "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/data/k2/manifest"
    wav_scp_path = "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/data/scp/wav.scp"
    text_scp_path = "/Users/xuelonggeng/Desktop/xlgeng_workspace/gxl_ai_utils/eggs/cats_and_dogs/rir_shujuzengguang/data/scp/text"
    utils_file.do_make_data4icefall(wav_scp_path, text_scp_path, fbank_dir=fbank_dir, manifest_dir=manifest_dir)





# Load model directly
from transformers import AutoTokenizer, AutoModelForCausalLM

tokenizer = AutoTokenizer.from_pretrained("Qwen/Qwen2-7B-Instruct")
model = AutoModelForCausalLM.from_pretrained("Qwen/Qwen2-7B-Instruct")
print(model)


import argparse
import json

from gxl_ai_utils.utils import utils_file
from sklearn.metrics import classification_report, accuracy_score, recall_score

def evaluate_emotion_prediction(list_file_path, text_file_path):
    """
    评估情感预测的性能，包括 Weighted Accuracy (WA), Unweighted Accuracy (UA), 和 Weighted F1 (WF1)。
    同时处理没有预测标签的样本，并统计未找到预测标签的数量。

    参数:
        list_file_path (str): data.list 文件的路径。
        text_file_path (str): text.txt 文件的路径。

    返回:
        dict: 包含 WA, UA, WF1 的计算结果和未找到预测标签的样本数量。
    """
    # 读取 data.list 文件
    with open(list_file_path, "r") as f:
        data_list = [json.loads(line.strip()) for line in f]

    # 读取 text.txt 文件
    with open(text_file_path, "r") as f:
        text_data = [line.strip().split("\t") for line in f]

    # 构造预测字典
    pred_dict = {}
    for line in text_data:
        if len(line) > 1 and "<" in line[1] and ">" in line[1]:
            pred_label = line[1].split("<")[-1].split(">")[0].upper()
            pred_dict[line[0]] = pred_label
        else:
            # 打印无效行的 key（如果存在）
            key = line[0] if len(line) > 0 else "UNKNOWN"
            print(f"Skipping invalid key: {key}")

    # 提取原始标签和预测标签
    y_true = []
    y_pred = []
    missing_keys = 0  # 统计没有预测标签的样本数量

    for entry in data_list:
        key = entry["key"]
        if key in pred_dict:
            y_true.append(entry["emotion"])
            y_pred.append(pred_dict[key])
        else:
            # 如果没有预测标签，统计并跳过
            missing_keys += 1

    if not y_true or not y_pred:
        raise ValueError("No valid predictions found. Check the input files.")

    # 计算 Weighted Accuracy (WA)
    wa = accuracy_score(y_true, y_pred)

    # 计算 Unweighted Accuracy (UA)
    ua = recall_score(y_true, y_pred, average="macro")
    unique_emotions = set(y_true)
    # ua = sum(
    #     accuracy_score(
    #         [true for true, pred in zip(y_true, y_pred) if true == emotion],
    #         [pred for true, pred in zip(y_true, y_pred) if true == emotion],
    #     )
    #     for emotion in unique_emotions
    # ) / len(unique_emotions)

    # 计算 Weighted F1 (WF1)
    report = classification_report(
        y_true, y_pred, labels=list(unique_emotions), output_dict=True
    )
    wf1 = sum(
        report[emotion]["f1-score"] * (y_true.count(emotion) / len(y_true))
        for emotion in unique_emotions if emotion in report
    )

    return {"WA": wa, "UA": ua, "WF1": wf1, "missing_keys": missing_keys}

# 主函数
if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    # parser.add_argument('--ref_file', type=str, required=True, help='reference file, 真实的标签')
    parser.add_argument('--hyp_file', type=str, required=True, help='hypothesis file， 推理的标签')
    parser.add_argument('--output_file', type=str, required=True, help='output file')
    args = parser.parse_args()

    # 用户输入文件路径
    list_file_path = "/home/work_nfs11/zxzhao/emotion.list"
    # text_file_path = "/home/work_nfs11/zxzhao/text"
    text_file_path = args.hyp_file

    # 调用函数并输出结果
    try:
        results = evaluate_emotion_prediction(list_file_path, text_file_path)
        print(f"Weighted Accuracy (WA): {results['WA']:.4f}")
        print(f"Unweighted Accuracy (UA): {results['UA']:.4f}")
        print(f"Weighted F1 (WF1): {results['WF1']:.4f}")
        print(f"未找到预测标签的样本数量: {results['missing_keys']}")
        res_list = [f"Weighted Accuracy (WA): {results['WA']:.4f}", f"Unweighted Accuracy (UA): {results['UA']:.4f}",
                    f"Weighted F1 (WF1): {results['WF1']:.4f}", f"未找到预测标签的样本数量: {results['missing_keys']}"]
        utils_file.write_list_to_file(res_list, args.output_file)
    except Exception as e:
        print(f"发生错误：{e}")


#!/usr/bin/env python3
# flake8: noqa
# Copyright      2022-2023  Xiaomi Corp.        (authors: Fangjun Kuang, Zengwei Yao)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
This script loads torchscript models exported by `torch.jit.script()`
and uses them to decode waves.
You can use the following command to get the exported models:

./zipformer/export.py \
  --exp-dir ./zipformer/exp \
  --causal 1 \
  --chunk-size 16 \
  --left-context-frames 128 \
  --tokens data/lang_bpe_500/tokens.txt \
  --epoch 30 \
  --avg 9 \
  --jit 1

Usage of this script:

./zipformer/jit_pretrained_streaming.py \
  --nn-model-filename ./zipformer/exp-causal/jit_script_chunk_16_left_128.pt \
  --tokens ./data/lang_bpe_500/tokens.txt \
  /path/to/foo.wav \
"""

import argparse
import logging
import math
from typing import List, Optional

import k2
import kaldifeat
import torch
import torchaudio
from kaldifeat import FbankOptions, OnlineFbank, OnlineFeature
from torch.nn.utils.rnn import pad_sequence


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    parser.add_argument(
        "--nn-model-filename",
        type=str,
        required=True,
        help="Path to the torchscript model jit_script.pt",
    )

    parser.add_argument(
        "--tokens",
        type=str,
        help="""Path to tokens.txt.""",
    )

    parser.add_argument(
        "--sample-rate",
        type=int,
        default=16000,
        help="The sample rate of the input sound file",
    )

    parser.add_argument(
        "sound_file",
        type=str,
        help="The input sound file(s) to transcribe. "
        "Supported formats are those supported by torchaudio.load(). "
        "For example, wav and flac are supported. "
        "The sample rate has to be 16kHz.",
    )

    return parser


def read_sound_files(
    filenames: List[str], expected_sample_rate: float
) -> List[torch.Tensor]:
    """Read a list of sound files into a list 1-D float32 torch tensors.
    Args:
      filenames:
        A list of sound filenames.
      expected_sample_rate:
        The expected sample rate of the sound files.
    Returns:
      Return a list of 1-D float32 torch tensors.
    """
    ans = []
    for f in filenames:
        wave, sample_rate = torchaudio.load(f)
        assert (
            sample_rate == expected_sample_rate
        ), f"expected sample rate: {expected_sample_rate}. Given: {sample_rate}"
        # We use only the first channel
        ans.append(wave[0])
    return ans


def greedy_search(
    decoder: torch.jit.ScriptModule,
    joiner: torch.jit.ScriptModule,
    encoder_out: torch.Tensor,
    decoder_out: Optional[torch.Tensor] = None,
    hyp: Optional[List[int]] = None,
    device: torch.device = torch.device("cpu"),
):
    assert encoder_out.ndim == 2
    context_size = decoder.context_size
    blank_id = decoder.blank_id

    if decoder_out is None:
        assert hyp is None, hyp
        hyp = [blank_id] * context_size
        decoder_input = torch.tensor(hyp, dtype=torch.int32, device=device).unsqueeze(0)
        # decoder_input.shape (1,, 1 context_size)
        decoder_out = decoder(decoder_input, torch.tensor([False])).squeeze(1)
    else:
        assert decoder_out.ndim == 2
        assert hyp is not None, hyp

    T = encoder_out.size(0)
    for i in range(T):
        cur_encoder_out = encoder_out[i : i + 1]
        joiner_out = joiner(cur_encoder_out, decoder_out).squeeze(0)
        y = joiner_out.argmax(dim=0).item()

        if y != blank_id:
            hyp.append(y)
            decoder_input = hyp[-context_size:]

            decoder_input = torch.tensor(
                decoder_input, dtype=torch.int32, device=device
            ).unsqueeze(0)
            decoder_out = decoder(decoder_input, torch.tensor([False])).squeeze(1)

    return hyp, decoder_out


def create_streaming_feature_extractor(sample_rate) -> OnlineFeature:
    """Create a CPU streaming feature extractor.

    At present, we assume it returns a fbank feature extractor with
    fixed options. In the future, we will support passing in the options
    from outside.

    Returns:
      Return a CPU streaming feature extractor.
    """
    opts = FbankOptions()
    opts.device = "cpu"
    opts.frame_opts.dither = 0
    opts.frame_opts.snip_edges = False
    opts.frame_opts.samp_freq = sample_rate
    opts.mel_opts.num_bins = 80
    opts.mel_opts.high_freq = -400
    return OnlineFbank(opts)


@torch.no_grad()
def main():
    parser = get_parser()
    args = parser.parse_args()
    logging.info(vars(args))

    device = torch.device("cpu")
    if torch.cuda.is_available():
        device = torch.device("cuda", 0)

    logging.info(f"device: {device}")

    model = torch.jit.load(args.nn_model_filename)
    model.eval()
    model.to(device)

    encoder = model.encoder
    decoder = model.decoder
    joiner = model.joiner

    token_table = k2.SymbolTable.from_file(args.tokens)
    context_size = decoder.context_size

    logging.info("Constructing Fbank computer")
    online_fbank = create_streaming_feature_extractor(args.sample_rate)

    logging.info(f"Reading sound files: {args.sound_file}")
    wave_samples = read_sound_files(
        filenames=[args.sound_file],
        expected_sample_rate=args.sample_rate,
    )[0]
    logging.info(wave_samples.shape)

    logging.info("Decoding started")

    chunk_length = encoder.chunk_size * 2
    T = chunk_length + encoder.pad_length

    logging.info(f"chunk_length: {chunk_length}")
    logging.info(f"T: {T}")

    states = encoder.get_init_states(device=device)

    tail_padding = torch.zeros(int(0.3 * args.sample_rate), dtype=torch.float32)

    wave_samples = torch.cat([wave_samples, tail_padding])

    chunk = int(0.25 * args.sample_rate)  # 0.2 second
    num_processed_frames = 0

    hyp = None
    decoder_out = None

    start = 0
    while start < wave_samples.numel():
        logging.info(f"{start}/{wave_samples.numel()}")
        end = min(start + chunk, wave_samples.numel())
        samples = wave_samples[start:end]
        start += chunk
        online_fbank.accept_waveform(
            sampling_rate=args.sample_rate,
            waveform=samples,
        )
        while online_fbank.num_frames_ready - num_processed_frames >= T:
            frames = []
            for i in range(T):
                frames.append(online_fbank.get_frame(num_processed_frames + i))
            frames = torch.cat(frames, dim=0).to(device).unsqueeze(0)
            x_lens = torch.tensor([T], dtype=torch.int32, device=device)
            encoder_out, out_lens, states = encoder(
                features=frames,
                feature_lengths=x_lens,
                states=states,
            )
            num_processed_frames += chunk_length

            hyp, decoder_out = greedy_search(
                decoder, joiner, encoder_out.squeeze(0), decoder_out, hyp, device=device
            )

    text = ""
    for i in hyp[context_size:]:
        text += token_table[i]
    text = text.replace("▁", " ").strip()

    logging.info(args.sound_file)
    logging.info(text)

    logging.info("Decoding Done")


torch.set_num_threads(4)
torch.set_num_interop_threads(1)
torch._C._jit_set_profiling_executor(False)
torch._C._jit_set_profiling_mode(False)
torch._C._set_graph_executor_optimize(False)
if __name__ == "__main__":
    formatter = "%(asctime)s %(levelname)s [%(filename)s:%(lineno)d] %(message)s"

    logging.basicConfig(format=formatter, level=logging.INFO)
    main()



from typing import Optional

import torch
from torch import nn
from wenet.utils.class_utils import WENET_ACTIVATION_CLASSES


class TransducerJoint(torch.nn.Module):

    def __init__(self,
                 vocab_size: int,
                 enc_output_size: int,
                 pred_output_size: int,
                 join_dim: int,
                 prejoin_linear: bool = True,
                 postjoin_linear: bool = False,
                 joint_mode: str = 'add',
                 activation: str = "tanh",
                 hat_joint: bool = False,
                 dropout_rate: float = 0.1,
                 hat_activation: str = 'tanh'):
        # TODO(Mddct): concat in future
        assert joint_mode in ['add']
        super().__init__()

        self.activatoin = WENET_ACTIVATION_CLASSES[activation]()
        self.prejoin_linear = prejoin_linear
        self.postjoin_linear = postjoin_linear
        self.joint_mode = joint_mode

        if not self.prejoin_linear and not self.postjoin_linear:
            assert enc_output_size == pred_output_size == join_dim
        # torchscript compatibility
        self.enc_ffn: Optional[nn.Linear] = None
        self.pred_ffn: Optional[nn.Linear] = None
        if self.prejoin_linear:
            self.enc_ffn = nn.Linear(enc_output_size, join_dim)
            self.pred_ffn = nn.Linear(pred_output_size, join_dim)
        # torchscript compatibility
        self.post_ffn: Optional[nn.Linear] = None
        if self.postjoin_linear:
            self.post_ffn = nn.Linear(join_dim, join_dim)

        # NOTE: <blank> in vocab_size
        self.hat_joint = hat_joint
        self.vocab_size = vocab_size
        self.ffn_out: Optional[torch.nn.Linear] = None
        if not self.hat_joint:
            self.ffn_out = nn.Linear(join_dim, vocab_size)

        self.blank_pred: Optional[torch.nn.Module] = None
        self.token_pred: Optional[torch.nn.Module] = None
        if self.hat_joint:
            self.blank_pred = torch.nn.Sequential(
                torch.nn.Tanh(), torch.nn.Dropout(dropout_rate),
                torch.nn.Linear(join_dim, 1), torch.nn.LogSigmoid())
            self.token_pred = torch.nn.Sequential(
                WENET_ACTIVATION_CLASSES[hat_activation](),
                torch.nn.Dropout(dropout_rate),
                torch.nn.Linear(join_dim, self.vocab_size - 1))

    def forward(self,
                enc_out: torch.Tensor,
                pred_out: torch.Tensor,
                pre_project: bool = True) -> torch.Tensor:
        """
        Args:
            enc_out (torch.Tensor): [B, T, E]
            pred_out (torch.Tensor): [B, T, P]
        Return:
            [B,T,U,V]
        """
        if (pre_project and self.prejoin_linear and self.enc_ffn is not None
                and self.pred_ffn is not None):
            enc_out = self.enc_ffn(enc_out)  # [B,T,E] -> [B,T,D]
            pred_out = self.pred_ffn(pred_out)
        if enc_out.ndim != 4:
            enc_out = enc_out.unsqueeze(2)  # [B,T,D] -> [B,T,1,D]
        if pred_out.ndim != 4:
            pred_out = pred_out.unsqueeze(1)  # [B,U,D] -> [B,1,U,D]

        # TODO(Mddct): concat joint
        _ = self.joint_mode
        out = enc_out + pred_out  # [B,T,U,V]

        if self.postjoin_linear and self.post_ffn is not None:
            out = self.post_ffn(out)

        if not self.hat_joint and self.ffn_out is not None:
            out = self.activatoin(out)
            out = self.ffn_out(out)
            return out
        else:
            assert self.blank_pred is not None
            assert self.token_pred is not None
            blank_logp = self.blank_pred(out)  # [B,T,U,1]

            # scale blank logp
            scale_logp = torch.clamp(1 - torch.exp(blank_logp), min=1e-6)
            label_logp = self.token_pred(out).log_softmax(
                dim=-1)  # [B,T,U,vocab-1]
            # scale token logp
            label_logp = torch.log(scale_logp) + label_logp

            out = torch.cat((blank_logp, label_logp), dim=-1)  # [B,T,U,vocab]
            return out



from transformers import AutoModelForCausalLM, AutoTokenizer
import torch
import os
os.environ["CUDA_VISIBLE_DEVICES"] = "5"
os.environ["HF_HOME"] = "./output/models-2B"
torch.manual_seed(0)

path = 'openbmb/MiniCPM-2B-sft-fp32'
path = "/home/work_nfs8/xlgeng/.cache/models--openbmb--MiniCPM-2B-sft-fp32/snapshots/35b90dd57d977b6e5bc4907986fa5b77aa15a82e"
print('hahaha')
tokenizer = AutoTokenizer.from_pretrained(path,trust_remote_code=True)
model = AutoModelForCausalLM.from_pretrained(path, torch_dtype=torch.float32, device_map='auto', trust_remote_code=True)

print('input: 山东省最高的山是哪座山, 它比黄山高还是矮？差距多少？')
responds, history = model.chat(tokenizer, "山东省最高的山是哪座山, 它比黄山高还是矮？差距多少？", temperature=0.8, top_p=0.8)
print(responds)
print('input:翻译如下英文至中文:China is an ancient country with a long history of civilization')
responds, history = model.chat(tokenizer, "翻译如下英文至中文:China is an ancient country with a long history of civilization", temperature=0.8, top_p=0.8)
print(responds)
print('input:翻译如下英文到中文:莎士比亚的文本具有一种高雅的美感')
responds, history = model.chat(tokenizer, "翻译如下英文到中文:莎士比亚的文本具有一种高雅的美感", temperature=0.8, top_p=0.8)
print(responds)
print('input:翻译如下句子到中文:我有一个pencil和ruler,你呢?')
responds, history = model.chat(tokenizer, "翻译如下句子到中文:我有一个pencil和ruler,你呢?", temperature=0.8, top_p=0.8)
print(responds)
print('input:翻译如下句子到英文:我有一个pencil和ruler,你呢?')
responds, history = model.chat(tokenizer, "翻译如下句子到英文:我有一个pencil和ruler,你呢?", temperature=0.8, top_p=0.8)
print(responds)


import os

input_path= "https://emilia-dataset.github.io/Emilia-Demo-Page/"
xpath_wav = "/html/body/div[1]/div/div/div[8]/div/div[2]/div/div[2]/div[1]/div[2]/div[1]/audio/source/@src"
language_xpath = "/html/body/div[1]/div/div/div[8]/div/div[2]/div/div[1]/text()"
l =              "/html/body/div[1]/div/div/div[8]/div/div[7]/div/div[1]/text()"
text_xpath = '/html/body/div[1]/div/div/div[8]/div/div[2]/div/div[2]/div[1]/div[1]/div/text()'
t =          '/html/body/div[1]/div/div/div[8]/div/div[7]/div/div[2]/div[1]/div[1]/text()'
from gxl_ai_utils.utils import utils_file, utils_spider
html_text_str = utils_spider.get_source_page_from_url(input_path)
utils_spider.write_to_html(html_text_str[0], 'text.html')
html_text_str = utils_spider.load_from_html('./text.html')

# for i in range(2,8):
#     """"""
#     output_dir = './data/emilia_demo'
#     output_dir_1 = os.path.join(output_dir, "demo_1")
#     os.makedirs(output_dir_1, exist_ok=True)
#     utils_file.write_list_to_file(['Samples generated by models trained with the full Emilia (101k hours) multilingual dataset.'], os.path.join(output_dir_1, f'title.txt'))
#
#     language_xpath = f"/html/body/div[1]/div/div/div[8]/div/div[{i}]/div/div[1]/text()"
#     language_i = utils_spider.handle_xpath(html_text_str, language_xpath)
#     print(language_i)
#
#     text_xpath = f'/html/body/div[1]/div/div/div[8]/div/div[{i}]/div/div[2]/div[1]/div[1]/div/text()'
#     text_i = utils_spider.handle_xpath(html_text_str, text_xpath)
#     print(text_i)
#     language_dir_i = os.path.join(output_dir_1, f'{language_i[0]}')
#     os.makedirs(language_dir_i, exist_ok=True)
#     utils_file.write_list_to_file([text_i[0]], os.path.join(language_dir_i, f'Target_Text:.txt'))
#     for j in range(1,4):
#         wav_xpath = f'/html/body/div[1]/div/div/div[8]/div/div[{i}]/div/div[2]/div[1]/div[2]/div[{j}]/audio/source/@src'
#         wav_i = utils_spider.handle_xpath(html_text_str, wav_xpath)
#         wav_path = f'https://emilia-dataset.github.io/Emilia-Demo-Page/{wav_i[0]}'
#         if j == 1:
#             file_name = "Speech_Prompt"
#         elif j == 2:
#             file_name = "SoundStorm"
#         else:
#             file_name = "VoiceBox"
#         utils_spider.download_file(wav_path, os.path.join(language_dir_i, f'{file_name}.wav'))

output_dir = "../data/emilia_demo/demo_2"
os.makedirs(output_dir, exist_ok=True)
for i in range(2,5):
    output_dir_i = os.path.join(output_dir, f"row_{i-1}")
    xpath_text = f"/html/body/div[1]/div/div/div[9]/div/div[{i}]/div[1]/div/text()"
    text = utils_spider.handle_xpath(html_text_str, xpath_text)
    utils_file.write_list_to_file([text[0]], os.path.join(output_dir_i, f'Target_Text.txt'))
    for j in range(1,6):
        wav_xpath = f"/html/body/div[1]/div/div/div[9]/div/div[{i}]/div[2]/div[{j}]/audio/source/@src"
        wav_i = utils_spider.handle_xpath(html_text_str, wav_xpath)
        wav_path = f'https://emilia-dataset.github.io/Emilia-Demo-Page/{wav_i[0]}'
        print(wav_path)
        if j == 1:
            filename = 'Speech_Prompt'
        elif j == 2:
            filename = 'SoundStorm-Emilia'
        elif j == 3:
            filename = 'SoundStorm-MLS'
        elif j == 4:
            filename = 'VoiceBox-Emilia'
        elif j == 5:
            filename = 'VoiceBox-MLS'
        utils_spider.download_file(wav_path, os.path.join(output_dir_i, f'{filename}.wav'))




from gxl_ai_utils.utils import utils_file

user_name = 'aslp'
password = "ASLPaslp"
remote_host = "10.68.109.101"
path_list = "/home/work_nfs14/pkchen/data/asr_token/asr/shards_list.txt"
output_dir = "/home/node54_tmpdata2/data4understand/asr_token"
utils_file.makedir_sil(output_dir)
utils_file.do_sync_files(
    file_list_path=path_list,
    password=password,
    username=user_name,
    remote_host=remote_host,
    local_directory=output_dir,
)
shards_list_path = f'{output_dir}/shards_list.txt'
utils_file.do_get_list_for_wav_dir(
    output_dir,shards_list_path, suffix='.tar'
)



# # Copyright (c) 2020 Mobvoi Inc. (authors: Binbin Zhang, Di Wu)
# #
# # Licensed under the Apache License, Version 2.0 (the "License");
# # you may not use this file except in compliance with the License.
# # You may obtain a copy of the License at
# #
# #     http://www.apache.org/licenses/LICENSE-2.0
# #
# # Unless required by applicable law or agreed to in writing, software
# # distributed under the License is distributed on an "AS IS" BASIS,
# # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# # See the License for the specific language governing permissions and
# # limitations under the License.
# # Modified from ESPnet(https://github.com/espnet/espnet)
# """Positonal Encoding Module."""

# import math
# from typing import Tuple, Union

# import torch
# import torch.nn.functional as F
# import numpy as np


# class PositionalEncoding(torch.nn.Module):
#     """Positional encoding.

#     :param int d_model: embedding dim
#     :param float dropout_rate: dropout rate
#     :param int max_len: maximum input length

#     PE(pos, 2i)   = sin(pos/(10000^(2i/dmodel)))
#     PE(pos, 2i+1) = cos(pos/(10000^(2i/dmodel)))
#     """

#     def __init__(self,
#                  d_model: int,
#                  dropout_rate: float,
#                  max_len: int = 5000,
#                  reverse: bool = False):
#         """Construct an PositionalEncoding object."""
#         super().__init__()
#         self.d_model = d_model
#         self.xscale = math.sqrt(self.d_model)
#         self.dropout = torch.nn.Dropout(p=dropout_rate)
#         self.max_len = max_len

#         self.pe = torch.zeros(self.max_len, self.d_model)
#         position = torch.arange(0, self.max_len,
#                                 dtype=torch.float32).unsqueeze(1)
#         div_term = torch.exp(
#             torch.arange(0, self.d_model, 2, dtype=torch.float32) *
#             -(math.log(10000.0) / self.d_model))
#         self.pe[:, 0::2] = torch.sin(position * div_term)
#         self.pe[:, 1::2] = torch.cos(position * div_term)
#         self.pe = self.pe.unsqueeze(0)

#     def forward(self,
#                 x: torch.Tensor,
#                 offset: Union[int, torch.Tensor] = 0) \
#             -> Tuple[torch.Tensor, torch.Tensor]:
#         """Add positional encoding.

#         Args:
#             x (torch.Tensor): Input. Its shape is (batch, time, ...)
#             offset (int, torch.tensor): position offset

#         Returns:
#             torch.Tensor: Encoded tensor. Its shape is (batch, time, ...)
#             torch.Tensor: for compatibility to RelPositionalEncoding
#         """

#         self.pe = self.pe.to(x.device)
#         pos_emb = self.position_encoding(offset, x.size(1), False)
#         x = x * self.xscale + pos_emb
#         return self.dropout(x), self.dropout(pos_emb)

#     def position_encoding(self,
#                           offset: Union[int, torch.Tensor],
#                           size: int,
#                           apply_dropout: bool = True) -> torch.Tensor:
#         """ For getting encoding in a streaming fashion

#         Attention!!!!!
#         we apply dropout only once at the whole utterance level in a none
#         streaming way, but will call this function several times with
#         increasing input size in a streaming scenario, so the dropout will
#         be applied several times.

#         Args:
#             offset (int or torch.tensor): start offset
#             size (int): required size of position encoding

#         Returns:
#             torch.Tensor: Corresponding encoding
#         """
#         # How to subscript a Union type:
#         #   https://github.com/pytorch/pytorch/issues/69434
#         if isinstance(offset, int):
#             assert offset + size <= self.max_len
#             pos_emb = self.pe[:, offset:offset + size]
#         elif isinstance(offset, torch.Tensor) and offset.dim() == 0:  # scalar
#             assert offset + size <= self.max_len
#             pos_emb = self.pe[:, offset:offset + size]
#         else:  # for batched streaming decoding on GPU
#             assert torch.max(offset) + size <= self.max_len
#             index = offset.unsqueeze(1) + \
#                 torch.arange(0, size).to(offset.device)  # B X T
#             flag = index > 0
#             # remove negative offset
#             index = index * flag
#             pos_emb = F.embedding(index, self.pe[0])  # B X T X d_model

#         if apply_dropout:
#             pos_emb = self.dropout(pos_emb)
#         return pos_emb


# class RelPositionalEncoding(PositionalEncoding):
#     """Relative positional encoding module.
#     See : Appendix B in https://arxiv.org/abs/1901.02860
#     Args:
#         d_model (int): Embedding dimension.
#         dropout_rate (float): Dropout rate.
#         max_len (int): Maximum input length.
#     """

#     def __init__(self, d_model: int, dropout_rate: float, max_len: int = 5000):
#         """Initialize class."""
#         super().__init__(d_model, dropout_rate, max_len, reverse=True)

#     def forward(self,
#                 x: torch.Tensor,
#                 offset: Union[int, torch.Tensor] = 0) \
#             -> Tuple[torch.Tensor, torch.Tensor]:
#         """Compute positional encoding.
#         Args:
#             x (torch.Tensor): Input tensor (batch, time, `*`).
#         Returns:
#             torch.Tensor: Encoded tensor (batch, time, `*`).
#             torch.Tensor: Positional embedding tensor (1, time, `*`).
#         """
#         self.pe = self.pe.to(x.device)
#         x = x * self.xscale
#         pos_emb = self.position_encoding(offset, x.size(1), False)
#         return self.dropout(x), self.dropout(pos_emb)


# class WhisperPositionalEncoding(PositionalEncoding):
#     """ Sinusoids position encoding used in openai-whisper.encoder
#     """

#     def __init__(self, d_model: int, dropout_rate: float, max_len: int = 1500):
#         super().__init__(d_model, dropout_rate, max_len)
#         self.xscale = 1.0
#         log_timescale_increment = np.log(10000) / (d_model // 2 - 1)
#         inv_timescales = torch.exp(-log_timescale_increment *
#                                    torch.arange(d_model // 2))
#         scaled_time = torch.arange(max_len)[:, np.newaxis] * \
#             inv_timescales[np.newaxis, :]
#         pe = torch.cat([torch.sin(scaled_time), torch.cos(scaled_time)], dim=1)
#         delattr(self, "pe")
#         self.register_buffer("pe", pe.unsqueeze(0))


# class LearnablePositionalEncoding(PositionalEncoding):
#     """ Learnable position encoding used in openai-whisper.decoder
#     """

#     def __init__(self, d_model: int, dropout_rate: float, max_len: int = 448):
#         super().__init__(d_model, dropout_rate, max_len)
#         # NOTE(xcsong): overwrite self.pe & self.xscale
#         self.pe = torch.nn.Parameter(torch.empty(1, max_len, d_model))
#         self.xscale = 1.0


# class NoPositionalEncoding(torch.nn.Module):
#     """ No position encoding
#     """

#     def __init__(self, d_model: int, dropout_rate: float):
#         super().__init__()
#         self.d_model = d_model
#         self.dropout = torch.nn.Dropout(p=dropout_rate)

#     def forward(self,
#                 x: torch.Tensor,
#                 offset: Union[int, torch.Tensor] = 0) \
#             -> Tuple[torch.Tensor, torch.Tensor]:
#         """ Just return zero vector for interface compatibility
#         """
#         pos_emb = torch.zeros(1, x.size(1), self.d_model).to(x.device)
#         return self.dropout(x), pos_emb

#     def position_encoding(self, offset: Union[int, torch.Tensor],
#                           size: int) -> torch.Tensor:
#         return torch.zeros(1, size, self.d_model)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2019 Mobvoi Inc. All Rights Reserved.
# Author: di.wu@mobvoi.com (DI WU)
"""Positonal Encoding Module."""

import math
from typing import Tuple, Union
import torch
import torch.nn.functional as F
import numpy as np

class PositionalEncoding(torch.nn.Module):
    """Positional encoding.

    :param int d_model: embedding dim
    :param float dropout_rate: dropout rate
    :param int max_len: maximum input length

    PE(pos, 2i)   = sin(pos/(10000^(2i/dmodel)))
    PE(pos, 2i+1) = cos(pos/(10000^(2i/dmodel)))
    """
    def __init__(self,
                 d_model: int,
                 dropout_rate: float,
                 max_len: int = 5000,
                 reverse: bool = False):
        """Construct an PositionalEncoding object."""
        super().__init__()
        self.d_model = d_model
        self.xscale = math.sqrt(self.d_model)
        self.dropout = torch.nn.Dropout(p=dropout_rate)
        self.max_len = max_len

        self.pe = torch.zeros(self.max_len, self.d_model)
        position = torch.arange(0, self.max_len,
                                dtype=torch.float32).unsqueeze(1)
        div_term = torch.exp(
            torch.arange(0, self.d_model, 2, dtype=torch.float32) *
            -(math.log(10000.0) / self.d_model))
        self.pe[:, 0::2] = torch.sin(position * div_term)
        self.pe[:, 1::2] = torch.cos(position * div_term)
        self.pe = self.pe.unsqueeze(0)

    def forward(self,
                x: torch.Tensor,
                offset: int = 0) -> Tuple[torch.Tensor, torch.Tensor]:
        """Add positional encoding.

        Args:
            x (torch.Tensor): Input. Its shape is (batch, time, ...)
            offset (int): position offset

        Returns:
            torch.Tensor: Encoded tensor. Its shape is (batch, time, ...)
            torch.Tensor: for compatibility to RelPositionalEncoding
        """
        assert offset + x.size(1) < self.max_len
        self.pe = self.pe.to(x.device)
        pos_emb = self.pe[:, offset:offset + x.size(1)]
        x = x * self.xscale + pos_emb
        return self.dropout(x), self.dropout(pos_emb)

    def position_encoding(self, offset: int, size: int) -> torch.Tensor:
        """ For getting encoding in a streaming fashion

        Attention!!!!!
        we apply dropout only once at the whole utterance level in a none
        streaming way, but will call this function several times with
        increasing input size in a streaming scenario, so the dropout will
        be applied several times.

        Args:
            offset (int): start offset
            size (int): requried size of position encoding

        Returns:
            torch.Tensor: Corresponding encoding
        """
        assert offset + size < self.max_len
        return self.dropout(self.pe[:, offset:offset + size])


class RelPositionalEncoding(PositionalEncoding):
    """Relative positional encoding module.
    See : Appendix B in https://arxiv.org/abs/1901.02860
    Args:
        d_model (int): Embedding dimension.
        dropout_rate (float): Dropout rate.
        max_len (int): Maximum input length.
    """
    def __init__(self, d_model: int, dropout_rate: float, max_len: int = 5000):
        """Initialize class."""
        super().__init__(d_model, dropout_rate, max_len, reverse=True)

    def forward(self,
                x: torch.Tensor,
                offset: int = 0) -> Tuple[torch.Tensor, torch.Tensor]:
        """Compute positional encoding.
        Args:
            x (torch.Tensor): Input tensor (batch, time, `*`).
        Returns:
            torch.Tensor: Encoded tensor (batch, time, `*`).
            torch.Tensor: Positional embedding tensor (1, time, `*`).
        """
        assert offset + x.size(1) < self.max_len
        self.pe = self.pe.to(x.device)
        x = x * self.xscale
        # import pdb;pdb.set_trace()
        pos_emb = self.pe[:, offset:offset + x.size(1)]
        return self.dropout(x), self.dropout(pos_emb)



class WhisperPositionalEncoding(PositionalEncoding):
    """ Sinusoids position encoding used in openai-whisper.encoder
    """

    def __init__(self, d_model: int, dropout_rate: float, max_len: int = 1500):
        super().__init__(d_model, dropout_rate, max_len)
        self.xscale = 1.0
        log_timescale_increment = np.log(10000) / (d_model // 2 - 1)
        inv_timescales = torch.exp(-log_timescale_increment *
                                   torch.arange(d_model // 2))
        scaled_time = torch.arange(max_len)[:, np.newaxis] * \
            inv_timescales[np.newaxis, :]
        pe = torch.cat([torch.sin(scaled_time), torch.cos(scaled_time)], dim=1)
        delattr(self, "pe")
        self.register_buffer("pe", pe.unsqueeze(0))


class LearnablePositionalEncoding(PositionalEncoding):
    """ Learnable position encoding used in openai-whisper.decoder
    """

    def __init__(self, d_model: int, dropout_rate: float, max_len: int = 448):
        super().__init__(d_model, dropout_rate, max_len)
        # NOTE(xcsong): overwrite self.pe & self.xscale
        self.pe = torch.nn.Parameter(torch.empty(1, max_len, d_model))
        self.xscale = 1.0


class NoPositionalEncoding(torch.nn.Module):
    """ No position encoding
    """

    def __init__(self, d_model: int, dropout_rate: float):
        super().__init__()
        self.d_model = d_model
        self.dropout = torch.nn.Dropout(p=dropout_rate)

    def forward(self,
                x: torch.Tensor,
                offset: Union[int, torch.Tensor] = 0) \
            -> Tuple[torch.Tensor, torch.Tensor]:
        """ Just return zero vector for interface compatibility
        """
        pos_emb = torch.zeros(1, x.size(1), self.d_model).to(x.device)
        return self.dropout(x), pos_emb

    def position_encoding(self, offset: Union[int, torch.Tensor],
                          size: int) -> torch.Tensor:
        return torch.zeros(1, size, self.d_model)



#!/usr/bin/env python3
# Copyright (c)  2021  Xiaomi Corporation (authors: Fangjun Kuang)
#
# See ../../../../LICENSE for clarification regarding multiple authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import k2
import torch
from rnn_lm.dataset import LmDataset, LmDatasetCollate


def main():
    sentences = k2.RaggedTensor(
        [[0, 1, 2], [1, 0, 1], [0, 1], [1, 3, 0, 2, 0], [3], [0, 2, 1]]
    )
    words = k2.RaggedTensor([[3, 6], [2, 8, 9, 3], [5], [5, 6, 7, 8, 9]])

    num_sentences = sentences.dim0

    sentence_lengths = [0] * num_sentences
    for i in range(num_sentences):
        word_ids = sentences[i]

        # NOTE: If word_ids is a tensor with only 1 entry,
        # token_ids is a torch.Tensor
        token_ids = words[word_ids]
        if isinstance(token_ids, k2.RaggedTensor):
            token_ids = token_ids.values

        # token_ids is a 1-D tensor containing the BPE tokens
        # of the current sentence

        sentence_lengths[i] = token_ids.numel()

    sentence_lengths = torch.tensor(sentence_lengths, dtype=torch.int32)

    indices = torch.argsort(sentence_lengths, descending=True)
    sentences = sentences[indices.to(torch.int32)]
    sentence_lengths = sentence_lengths[indices]

    dataset = LmDataset(
        sentences=sentences,
        words=words,
        sentence_lengths=sentence_lengths,
        max_sent_len=3,
        batch_size=4,
    )

    collate_fn = LmDatasetCollate(sos_id=1, eos_id=-1, blank_id=0)
    dataloader = torch.utils.data.DataLoader(
        dataset, batch_size=1, collate_fn=collate_fn
    )

    for i in dataloader:
        print(i)
    # I've checked the output manually; the output is as expected.


if __name__ == "__main__":
    main()



from gxl_ai_utils.utils import utils_file

if __name__ == "__main__":
    """"""
    token_file = "/home/work_nfs7/yhliang/online_sys/wenet/examples/onlinesys/data/dict/lang_char.txt"
    word_file = "/home/work_nfs4_ssd/yhliang/LanguageModel/lang/lang_test616_mix0.1_pruned_5e-11/words.txt"
    arpa_path = '/home/work_nfs14/xlgeng/new_workspace/wenet_gxl_aishell/examples/aishell/s0/LM_training/data/output_all_data_temp.arpa'
    output_dir = "./output_data_2/"
    utils_file.do_make_HLG(word_file, token_file, arpa_path, output_dir)



import torch
from ..paraformer.cif import Cif
from ..paraformer.layers import (SanmDecoder, SanmEncoder)
from ..paraformer.paraformer import Paraformer
from ..transformer.cmvn import GlobalCMVN
from ..utils.checkpoint import load_checkpoint
from ..utils.cmvn import load_cmvn


def init_model(configs, checkpoint_path=None):
    mean, istd = load_cmvn(configs['cmvn_file'], True)
    global_cmvn = GlobalCMVN(
        torch.from_numpy(mean).float(),
        torch.from_numpy(istd).float())
    input_dim = configs['input_dim']
    vocab_size = configs['output_dim']
    encoder = SanmEncoder(global_cmvn=global_cmvn,
                          input_size=configs['lfr_conf']['lfr_m'] * input_dim,
                          **configs['encoder_conf'])
    decoder = SanmDecoder(vocab_size=vocab_size,
                          encoder_output_size=encoder.output_size(),
                          **configs['decoder_conf'])
    predictor = Cif(**configs['cif_predictor_conf'])
    model = Paraformer(
        vocab_size=vocab_size,
        encoder=encoder,
        decoder=decoder,
        predictor=predictor,
        **configs['model_conf'],
    )

    if checkpoint_path is not None:
        load_checkpoint(model, checkpoint_path)
    return model, configs
